# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__author__ = 'Erdinç Yılmaz'
__date__ = '3/28/16'

from PySide6.QtCore import Qt
from PySide6.QtGui import QPainterPath
from PySide6.QtWidgets import QStyle
from ..nesneler.base import BaseItem
from .. import shared


########################################################################
class Ellipse(BaseItem):
    Type = shared.ELLIPSE_ITEM_TYPE

    # ---------------------------------------------------------------------
    def __init__(self, pos, rect, yaziRengi, arkaPlanRengi, pen, font, parent=None):
        super(Ellipse, self).__init__(pos, rect, yaziRengi, arkaPlanRengi, pen, font, parent)

    # ---------------------------------------------------------------------
    def type(self):
        # Enable the use of qgraphicsitem_cast with this item.
        return Ellipse.Type

    # ---------------------------------------------------------------------
    def shape(self):
        path = QPainterPath()
        if self._rect.isNull():
            return path
        # path = super(Ellipse, self).shape()
        # path.addEllipse(self.boundingRect())

        path.addEllipse(self._rect)
        # path.addRect(self._rect)
        if not self.isPinned:
            path.addRect(self.topLeftHandle)
            path.addRect(self.topRightHandle)
            path.addRect(self.bottomRightHandle)
            path.addRect(self.bottomLeftHandle)
        # return path
        return self.qt_graphicsItem_shapeFromPath(path, self._pen)

    # ---------------------------------------------------------------------
    def boundingRect(self):
        return self.shape().controlPointRect()

    # TODO: self.shape().controlPointRect() normalde calisiyordu, fakat resize ile ilgili
    # setPos ve moveTo(0,0) ile alakali bazi yeni durumlar ortaya cikti su an gerek yok. o yuzden
    # iptal edildi, basedei boundingRect methodu gayet yeterli burda da. 
    # # ---------------------------------------------------------------------
    # def boundingRect(self):
    #
    #     if self._boundingRect.isNull():
    #         # simdilik alttaki blok yerine bu alttaki satir
    #         self._boundingRect = self.shape().controlPointRect()
    #
    #         # pw = self.pen().widthF()
    #         # if not pw:
    #         #     # normalde self_rect yeterli, ama handlelar disarda kaliyor
    #         #     # ellipse handle sizedan kucuk olunca, shape() icine de handlelari
    #         #     # dahil ettigimizden, shape().controlPointRect() calisiyor.
    #         #     # self._boundingRect = self._rect
    #         #     self._boundingRect = self.shape().controlPointRect()
    #         # else:
    #         #     self._boundingRect = self.shape().controlPointRect()
    #         #     # self.shape().boundingRect() daha yavaş contolPointe gore.
    #         #     # self._boundingRect = self.shape().boundingRect()
    #
    #     return self._boundingRect

    # ---------------------------------------------------------------------
    def paint(self, painter, option, widget=None):

        if not self._pen.width():
            painter.setPen(Qt.PenStyle.NoPen)
        else:
            painter.setPen(self._pen)
        painter.setBrush(self._brush)
        painter.drawEllipse(self._rect)
        # painter.drawEllipse(self.boundingRect())

        if self.text():
            painter.save()
            painter.setFont(self._font)
            painter.translate(self._rect.center())
            painter.rotate(-self.rotation())

            painter.translate(-self._rect.center())

            # ---------------------------------------------------------------------
            # --  basla --  elided text cizmek icin --
            # ---------------------------------------------------------------------
            # metrics = painter.fontMetrics()
            # text = metrics.elidedText(self._text, Qt.ElideRight, self._rect.width() / self.painterTextScale)
            # painter.drawText(self._rect, Qt.AlignCenter, text)
            # ---------------------------------------------------------------------
            # ---  bitir --- elided text cizmek icin --
            # ---------------------------------------------------------------------
            painter.setPen(self.yaziRengi)
            painter.drawText(self.painterTextRect, self._text, self.painterTextOption)

            painter.restore()
        # painter.setWorldMatrixEnabled(True)

        if option.state & QStyle.StateFlag.State_Selected or self.cosmeticSelect:

            if self.isActiveItem:
                selectionPenBottom = self.selectionPenBottomIfAlsoActiveItem
                if not self.isPinned:

                    painter.setPen(Qt.PenStyle.NoPen)
                    painter.setBrush(self.handleBrush)
                    painter.drawRect(self.topLeftHandle)
                    painter.drawRect(self.topRightHandle)
                    painter.drawRect(self.bottomRightHandle)
                    painter.drawRect(self.bottomLeftHandle)
            else:
                selectionPenBottom = self.selectionPenBottom

            painter.setBrush(Qt.BrushStyle.NoBrush)

            painter.setPen(selectionPenBottom)
            painter.drawEllipse(self._rect)

            # painter.setPen(self.selectionPenTop)
            # painter.drawEllipse(self._rect)

            if not self.isPinned:
            #     # painter.drawRect(self.boundingRect())
                painter.drawRect(self._rect)
            #
            #     painter.setPen(Qt.PenStyle.NoPen)
            #     painter.setBrush(self.handleBrush)
            #     # painter.setPen(self.selectionPenTop)
            #     # painter.drawRect(self.boundingRect())
            #     # painter.drawRect(self._rect)
            #
            #     # draw handles
            #     painter.drawRect(self.topLeftHandle)
            #     painter.drawRect(self.topRightHandle)
            #     painter.drawRect(self.bottomRightHandle)
            #     painter.drawRect(self.bottomLeftHandle)

        # if option.state & QStyle.StateFlag.State_MouseOver:
        #     painter.setBrush(Qt.BrushStyle.NoBrush)
        #     painter.setPen(self.selectionPenBottom)
        #     painter.drawEllipse(self._rect)

        # # # # # # debug start - pos() # # # # #
        # p = self.pos()
        # s = self.scenePos()
        # painter.drawText(self._rect,
        #                  "{0:.2f},  {1:.2f} pos \n{2:.2f},  {3:.2f} spos".format(p.x(), p.y(), s.x(), s.y()))
        # # # t = self.transformOriginPoint()
        # # # painter.drawRect(t.x()-12, t.y()-12,24,24)
        # mapped = self.mapToScene(self._rect.topLeft())
        # painter.drawText(self._rect.x(), self._rect.y(), "{0:.2f}  {1:.2f} map".format(mapped.x(), mapped.y()))
        # painter.drawEllipse(self.scenePos(), 10, 10)
        # painter.setPen(Qt.blue)
        # painter.drawEllipse(self.mapFromScene(self.pos()), 10, 10)
        # r = self.textItem.boundingRect()
        # r = self.mapRectFromItem(self.textItem, r)
        # painter.drawRect(r)
        # painter.drawText(self._rect.center(), "{0:f}  {1:f}".format(self.sceneWidth(), self.sceneHeight()))
        # painter.setPen(QPen(Qt.red,17))
        # painter.drawPoint(self._rect.center())
        # painter.setPen(QPen(Qt.green,12))
        # painter.drawPoint(self.mapFromScene(self.sceneBoundingRect().center()))
        # painter.setPen(QPen(Qt.blue,8))
        # painter.drawPoint(self.sceneBoundingRect().center())
        # painter.drawRect(self.sceneBoundingRect())
        # # # # # # debug end - pos() # # # # #

    # ---------------------------------------------------------------------
    def html_dive_cevir(self, html_klasor_kayit_adres, dosya_kopyalaniyor_mu):

        arkaPlanRengi = self.arkaPlanRengi.toRgb()
        yaziRengi = self.yaziRengi.toRgb()
        cizgiRengi = self.cizgiRengi.toRgb()

        w = self._rect.width()
        h = self._rect.height()

        c = self.sceneBoundingRect().center()

        xr = c.x() - w / 2
        yr = c.y() - h / 2
        xs = self.scene().sceneRect().x()
        ys = self.scene().sceneRect().y()
        x = xr - xs
        y = yr - ys

        bicimSozluk = self.ver_karakter_bicimi()
        bold = 'font-weight="bold" ' if bicimSozluk["b"] else ''
        italic = ' font-style="italic"' if bicimSozluk["i"] else ''
        underline = "underline" if bicimSozluk["u"] else ''
        strikeOut = "line-through" if bicimSozluk["s"] else ''
        overline = "overline" if bicimSozluk["o"] else ''
        bicimler1 = bold + italic
        if any((underline, strikeOut, overline)):
            bicimler2 = f'text-decoration= "{underline} {strikeOut} {overline}"'
        else:
            bicimler2 = ''

        hiza = self.ver_yazi_hizasi()
        # if hiza == Qt.AlignmentFlag.AlignLeft or hiza == Qt.AlignmentFlag.AlignLeft | Qt.AlignmentFlag.AlignVCenter:
        #     yazi_hiza = "left"
        if hiza == Qt.AlignmentFlag.AlignCenter or hiza == Qt.AlignmentFlag.AlignCenter | Qt.AlignmentFlag.AlignVCenter:
            yazi_hiza = "center"
        elif hiza == Qt.AlignmentFlag.AlignRight or hiza == Qt.AlignmentFlag.AlignRight | Qt.AlignmentFlag.AlignVCenter:
            yazi_hiza = "right"
        elif hiza == Qt.AlignmentFlag.AlignJustify or hiza == Qt.AlignmentFlag.AlignJustify | Qt.AlignmentFlag.AlignVCenter:
            yazi_hiza = "justify"
        else:
            yazi_hiza = "left"

        if self.rotation():
            dondur_str_eksi = f"""
            transform-box: fill-box;
            transform-origin: center;
            transform: rotate({-self.rotation()}deg);
            """
        else:
            dondur_str_eksi = ''

        # x ="{self.painterTextRect.center().x()}" y="{self.painterTextRect.center().y()}"
        # x ="%50" y="%50" dominant-baseline="middle" text-anchor="middle"
        if self.text():
            yazi_str = f"""
            <text 
            style="{dondur_str_eksi}"
            fill="rgba{yaziRengi.toTuple()}"
            fill-opacity="{yaziRengi.alpha() / 255}"
            stroke="none" xml:space="preserve" 
            text-anchor="middle"
            alignment-baseline="middle"
            x="{w / 2}"
            y="{h / 2}"
            font-family={self.font().family()}
            font-size="{self.fontPointSizeF()}pt"
            {bicimler1}
            {bicimler2}
            text-align="{yazi_hiza}"
             >{self.text()}</text>
            """
        else:
            yazi_str = ""

        ellipse_str = f"""
                  <ellipse
                    style="fill:rgba{arkaPlanRengi.toTuple()};
                    fill-opacity:{arkaPlanRengi.alpha() / 255};
                    stroke:rgba{cizgiRengi.toTuple()};
                    stroke-opacity:{cizgiRengi.alpha() / 255};
                    stroke-width:{self._pen.widthF() * 2};
                    stroke-linecap:round;
                    stroke-dasharray:none;
                    stroke-linejoin:round;
                    paint-order:markers fill stroke;"
                    cx="{w / 2}" cy="{h / 2}" rx="{w / 2 - self._pen.widthF()}" ry="{h / 2 - self._pen.widthF()}"
                   />
        """

        svg_str = f"""
        <?xml version="1.0"?>
            <svg xmlns="http://www.w3.org/2000/svg" version="1.2" baseProfile="tiny" 
             width="{w}" height="{h}" viewBox="0 0 {w} {h}" >
                {ellipse_str}
                {yazi_str}
              </svg>\n
        """

        div_str = f"""
                    <div style="
                     position:absolute;
                     z-index:{int(self.zValue() * 10) if self.zValue() else 0};
                     width:{w}px;
                     height:{h}px;
                     top:{y}px;
                     left:{x}px;
                     transform-box: fill-box;
                     transform-origin: center;
                     transform: rotate({self.rotation()}deg);"
                      id="{self._kim}">{svg_str}</div>\n
            """
        # return svg_str
        return div_str



# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__author__ = 'Erdinç Yılmaz'
__date__ = '3/28/16'

from PySide6.QtCore import Qt
from ..nesneler.base import BaseItem
from .. import shared


########################################################################
class Rect(BaseItem):
    Type = shared.RECT_ITEM_TYPE

    # ---------------------------------------------------------------------
    def __init__(self, pos, rect, yaziRengi, arkaPlanRengi, pen, font, parent=None):
        super(Rect, self).__init__(pos, rect, yaziRengi, arkaPlanRengi, pen, font, parent)

    # ---------------------------------------------------------------------
    def type(self):
        # Enable the use of qgraphicsitem_cast with this item.
        return Rect.Type

    # ---------------------------------------------------------------------
    def html_dive_cevir(self, html_klasor_kayit_adres, dosya_kopyalaniyor_mu):

        arkaPlanRengi = self.arkaPlanRengi.toRgb()
        yaziRengi = self.yaziRengi.toRgb()
        cizgiRengi = self.cizgiRengi.toRgb()

        w = self._rect.width()
        h = self._rect.height()

        c = self.sceneBoundingRect().center()

        xr = c.x() - w / 2
        yr = c.y() - h / 2
        xs = self.scene().sceneRect().x()
        ys = self.scene().sceneRect().y()
        x = xr - xs
        y = yr - ys

        bicimSozluk = self.ver_karakter_bicimi()
        bold = 'font-weight="bold" ' if bicimSozluk["b"] else ''
        italic = ' font-style="italic"' if bicimSozluk["i"] else ''
        underline = "underline" if bicimSozluk["u"] else ''
        strikeOut = "line-through" if bicimSozluk["s"] else ''
        overline = "overline" if bicimSozluk["o"] else ''
        bicimler1 = bold + italic
        if any((underline, strikeOut, overline)):
            bicimler2 = f'text-decoration= "{underline} {strikeOut} {overline}"'
        else:
            bicimler2 = ''

        hiza = self.ver_yazi_hizasi()
        # if hiza == Qt.AlignLeft or hiza == Qt.AlignLeft | Qt.AlignVCenter:
        #     yazi_hiza = "left"
        if hiza == Qt.AlignmentFlag.AlignCenter or hiza == Qt.AlignmentFlag.AlignCenter | Qt.AlignmentFlag.AlignVCenter:
            yazi_hiza = "center"
        elif hiza == Qt.AlignmentFlag.AlignRight or hiza == Qt.AlignmentFlag.AlignRight | Qt.AlignmentFlag.AlignVCenter:
            yazi_hiza = "right"
        elif hiza == Qt.AlignmentFlag.AlignJustify or hiza == Qt.AlignmentFlag.AlignJustify | Qt.AlignmentFlag.AlignVCenter:
            yazi_hiza = "justify"
        else:
            yazi_hiza = "left"

        if self.rotation():
            # dondur_str = f'transform="rotate({self.rotation()},{x+w/2},{y+h/2})"'
            dondur_str_eksi = f"""transform-box: fill-box;
                                  transform-origin: center;
                                  transform: rotate({-self.rotation()}deg);"""
            # dondur_str = f"""
            # transform-box: fill-box;
            # transform-origin: center;
            # transform: rotate({self.rotation()}deg);
            # """

        else:
            # dondur_str =''
            dondur_str_eksi = ''

        # x="{self.painterTextRect.center().x()}" y="{self.painterTextRect.center().y()}"
        # x="%50" y="%50" dominant-baseline="middle" text-anchor="middle" 
        if self.text():
            yazi_str = f"""
            <text style="{dondur_str_eksi}"
            fill="rgba{yaziRengi.toTuple()}" fill-opacity="{yaziRengi.alpha() / 255}"
            stroke="none" xml:space="preserve" 
            x="{w / 2}" y="{h / 2}" text-anchor="middle" alignment-baseline="middle"
            font-family={self.font().family()} font-size="{self.fontPointSizeF()}pt"
            {bicimler1} {bicimler2} text-align="{yazi_hiza}">{self.text()}</text>
            """
        else:
            yazi_str = ""

        rect_str = f"""
        <rect width="{w}" height="{h}" x="0" y="0"
                   style="fill:rgba{arkaPlanRengi.toTuple()};
                   fill-opacity:{arkaPlanRengi.alpha() / 255};
                   stroke:rgba{cizgiRengi.toTuple()};
                   stroke-opacity:{cizgiRengi.alpha() / 255};
                   stroke-width:{self._pen.widthF() * 2};
                   stroke-linecap:round; stroke-dasharray:none; stroke-linejoin:round;
                   paint-order:markers fill stroke;"/>
        """

        svg_str = f"""
        <?xml version="1.0"?>
        <svg xmlns="http://www.w3.org/2000/svg" version="1.2" baseProfile="tiny" 
            width="{w}" height="{h}" viewBox="0 0 {w} {h}" >
        {rect_str}
        {yazi_str}
        </svg>\n
        """

        div_str = f"""
        <div id="{self._kim}"
            style="position:absolute;
                   z-index:{int(self.zValue() * 10) if self.zValue() else 0};
                   width:{w}px; height:{h}px; top:{y}px; left:{x}px; 
                   transform-box: fill-box; 
                   transform-origin: center; 
                   transform: rotate({self.rotation()}deg);">{svg_str}</div>\n
        """
        # return svg_str
        return div_str



# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__author__ = 'Erdinç Yılmaz'
__date__ = '3/28/16'

from PySide6.QtGui import QPen
from PySide6.QtWidgets import QGraphicsLineItem, QGraphicsTextItem
from PySide6.QtCore import Qt
from .. import shared


########################################################################
class MirrorLine(QGraphicsLineItem):
    Type = shared.MIRRORLINE_TYPE

    # ---------------------------------------------------------------------
    def __init__(self, posFeedback, axis, parent=None):
        super(MirrorLine, self).__init__(parent)

        # self.setFlag(QGraphicsLineItem.ItemIgnoresTransformations, True)
        self._kim = shared.kim(kac_basamak=16)
        pen = QPen(Qt.GlobalColor.blue, 1, Qt.PenStyle.DashDotDotLine)
        self.setPen(pen)
        self.setZValue(10000)

        self.axis = axis
        self.textItem = QGraphicsTextItem()
        self.textItem.setParentItem(self)
        self.textItem.setPos(posFeedback)
        self.textItem.setDefaultTextColor(Qt.GlobalColor.blue)
        self.textItem.setPlainText("    {}: {}".format(self.axis, posFeedback.x()))

    # def setLine(self):
    # def paint(self, painter, option, widget=None):
    #     super(MirrorLine, self).paint(painter, option, widget)

    # ---------------------------------------------------------------------
    def updateScale(self):
        yeniOran = 1 / self.scene().views()[0].viewportTransform().m11()
        # init te pen kalinligi 1 dolayisi ile 1 / viewscale den gelen degeri direkt
        #  kalem kalinligina ve de textItem in scale ine uyguluyabiliyoruz.
        # ilk ve hep istenen deger baska olaydi, o degere bolmek gerekirdi o degeri cekip.
        pen = self.pen()
        pen.setWidthF(yeniOran)
        self.setPen(pen)
        self.textItem.setScale(yeniOran)

    # ---------------------------------------------------------------------
    def type(self):
        # Enable the use of qgraphicsitem_cast with this item.
        return MirrorLine.Type

    # ---------------------------------------------------------------------
    def updatePosFeedBack(self, posFeedback):
        self.textItem.setPos(posFeedback)
        if self.axis == "x":
            pos = posFeedback.x()
        else:
            pos = posFeedback.y()
        self.textItem.setPlainText("    {}: {}".format(self.axis, pos))

    # ---------------------------------------------------------------------
    def html_dive_cevir(self, html_klasor_kayit_adres, dosya_kopyalaniyor_mu):
        # Ola ki bu nesne aktif iken HTML olarak kaydet cagrilirsa diye
        # None dondurmeyelim stringe ekleniyor bu
        return ""



# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__author__ = 'Erdinç Yılmaz'
__date__ = '3/28/16'

from PySide6.QtCore import Qt, QRectF, QPointF, QSizeF
from PySide6.QtGui import QColor, QPen
from PySide6.QtWidgets import QStyle, QGraphicsItem
from .. import shared


########################################################################
class Group(QGraphicsItem):
    Type = shared.GROUP_ITEM_TYPE

    def __init__(self, arkaPlanRengi=QColor(Qt.GlobalColor.transparent),
                 yaziRengi=QColor(Qt.GlobalColor.transparent),
                 pen=QPen(Qt.PenStyle.DotLine),
                 parent=None):
        super(Group, self).__init__(parent)

        self._kim = shared.kim(kac_basamak=16)
        self.setAcceptHoverEvents(True)

        self.ustGrup = None
        # self._rect aslinda => self._itemsBoundingRect
        self._rect = QRectF()

        self.setHandlesChildEvents(False)
        self.setFlags(QGraphicsItem.GraphicsItemFlag.ItemIsSelectable
                      | QGraphicsItem.GraphicsItemFlag.ItemIsMovable
                      # | QGraphicsItem.GraphicsItemFlag.ItemIsFocusable
                      )

        self.secili_nesne_kalem_kalinligi = 0
        self.cizgiTipi = Qt.PenStyle.SolidLine
        self.cizgiUcuTipi = Qt.PenCapStyle.RoundCap
        self.cizgiBirlesimTipi = Qt.PenJoinStyle.RoundJoin

        self.activeItemLineColor = shared.activeItemLineColor
        self._pen = pen
        self.setCizgiRengi(pen.color())
        self.arkaPlanRengi = arkaPlanRengi  # grup nesnesinde kullanilmiyor
        self.yaziRengi = yaziRengi  # grup nesnesinde kullanilmiyor
        self.cizgiRengi = pen.color()

        self.cosmeticSelect = False
        self.isActiveItem = False
        self._isPinned = False

        self.allNonGroupGroupChildren = []
        self.parentedWithParentOperation = []  # as is
        self.allFirstLevelGroupChildren = []  # we use this for calculating group's bounding box.

        self.oklar_dxdy_nokta = {}

        self._eskiRectBeforeResize = None
        self._eskiPosBeforeResize = None

        self._resizing = False
        self.handleSize = 10
        self.resizeHandleSize = QSizeF(self.handleSize, self.handleSize)
        self.boundingRectTasmaDegeri = ((self._pen.widthF() / 2) + (self.handleSize / 2))

        self.update_resize_handles()

    # ---------------------------------------------------------------------
    def setRect(self, rect):
        if self._rect == rect:
            return
        self.prepareGeometryChange()
        self._rect = rect
        self.update_resize_handles()
        # self.update()

    # ---------------------------------------------------------------------
    def rect(self):
        return self._rect

    # ---------------------------------------------------------------------
    def update_resize_handles(self):
        self.prepareGeometryChange()

        if self.scene():
            resizeHandleSize = self.resizeHandleSize / self.scene().views()[0].transform().m11()
        else:
            resizeHandleSize = self.resizeHandleSize

        self.topLeftHandle = QRectF(self._rect.topLeft(), resizeHandleSize)
        self.topRightHandle = QRectF(self._rect.topRight(), resizeHandleSize)
        self.bottomRightHandle = QRectF(self._rect.bottomRight(), resizeHandleSize)
        self.bottomLeftHandle = QRectF(self._rect.bottomLeft(), resizeHandleSize)

        self.topLeftHandle.moveCenter(self._rect.topLeft())
        self.topRightHandle.moveCenter(self._rect.topRight())
        self.bottomRightHandle.moveCenter(self._rect.bottomRight())
        self.bottomLeftHandle.moveCenter(self._rect.bottomLeft())

    # ---------------------------------------------------------------------
    def hide_resize_handles(self):
        self.isPinned = True

    # ---------------------------------------------------------------------
    def show_resize_handles(self):
        self.isPinned = False

    # ---------------------------------------------------------------------
    def mousePressEvent(self, event):

        self._resizing = False  # we could use "self._resizingFrom = 0" instead, but self._resizing is more explicit.
        self._fixedResizePoint = None
        self._resizingFrom = None
        self._eskiRectBeforeResize = None

        self._eskiPosBeforeResize = self.pos()
        self._ilk_tik_event_pos = event.pos()
        # self._eskiPosBeforeResize = self.scenePos()

        if self.topLeftHandle.contains(event.pos()):
            self._resizing = True
            self._fixedResizePoint = self._rect.bottomRight()
            self._resizingFrom = 1
        elif self.topRightHandle.contains(event.pos()):
            self._resizing = True
            self._fixedResizePoint = self._rect.bottomLeft()
            self._resizingFrom = 2
        elif self.bottomRightHandle.contains(event.pos()):
            self._resizing = True
            self._fixedResizePoint = self._rect.topLeft()
            self._resizingFrom = 3
        elif self.bottomLeftHandle.contains(event.pos()):
            self._resizing = True
            self._fixedResizePoint = self._rect.topRight()
            self._resizingFrom = 4
        self._eskiRectBeforeResize = self._rect

        super(Group, self).mousePressEvent(event)

        # print(self.allFirstLevelGroupChildren)
        # print(self.allNonGroupGroupChildren)
        # print(self.parentedWithParentOperation)

        # TODO: bu kozmetik amacli, sadece release de olunca gecikmeli seciyormus hissi oluyor
        if self.isSelected() and self.parentedWithParentOperation:
            self.scene().select_all_children_recursively(self, cosmeticSelect=True)

        # self.scene().deactivate_item(self)
        # self.scene().activate_item(self)

    # ---------------------------------------------------------------------
    def mouseMoveEvent(self, event):

        # TODO: burda QRectF().normalized da kullanilabilir belki..

        if self._resizing:

            px = self._fixedResizePoint.x()
            py = self._fixedResizePoint.y()
            # mx = event.scenePos().x()
            # my = event.scenePos().y()
            mx = event.pos().x()
            my = event.pos().y()
            topLeft = QPointF(min(px, mx), min(py, my))  # top-left corner (x,y)
            bottomRight = QPointF(max(px, mx), max(py, my))  # bottom-right corner (x,y)
            # size = QSizeF(fabs(mx - px), fabs(my - py))
            # rect = QRectF(topLeft, size)

            rect = QRectF(topLeft, bottomRight)

            c = self._rect.center()

            # Alt Key - to resize around center.
            # if event.modifiers() & Qt.AltModifier:
            if event.modifiers() == Qt.KeyboardModifier.AltModifier:
                rect.moveCenter(c)

            # ---------------------------------------------------------------------
            #  Ctrl Key - to keep aspect ratio while resizing.
            if event.modifiers() == Qt.KeyboardModifier.ControlModifier:
                tl = self._rect.topLeft()
                tr = self._rect.topRight()
                br = self._rect.bottomRight()
                bl = self._rect.bottomLeft()
                c = self._rect.center()

                yeniSize = rect.size()
                eskiSize = self._rect.size()
                # eskiSize.scale(yeniSize, Qt.KeepAspectRatio)
                eskiSize.scale(yeniSize.height(), yeniSize.height(), Qt.AspectRatioMode.KeepAspectRatioByExpanding)
                # eskiSize.scale(yeniSize.height(), yeniSize.height(), Qt.KeepAspectRatio)

                # if not eskiSize.isNull():
                if not eskiSize.isEmpty():
                    self.yedekSize = QSizeF(eskiSize)

                else:
                    eskiSize = QSizeF(self.yedekSize)

                rect.setSize(eskiSize)
                h = rect.height()
                w = rect.width()

                if self._resizingFrom == 1:
                    rect.moveTopLeft(QPointF(br.x() - w, br.y() - h))

                if self._resizingFrom == 2:
                    rect.moveTopRight(QPointF(bl.x() + w, bl.y() - h))

                if self._resizingFrom == 3:
                    rect.moveBottomRight(QPointF(tl.x() + w, tl.y() + h))

                elif self._resizingFrom == 4:
                    rect.moveBottomLeft(QPointF(tr.x() - w, tr.y() + h))

                # Alt Key - to resize around center
                if event.modifiers() == Qt.KeyboardModifier.AltModifier:
                    rect.moveCenter(c)

            # ---------------------------------------------------------------------
            # Shift Key - to make square (equals width and height)
            if event.modifiers() == Qt.KeyboardModifier.ShiftModifier:
                height = my - py
                if self._resizingFrom == 1:
                    rect.setCoords(px + height, py + height, px, py)
                    rect = rect.normalized()

                elif self._resizingFrom == 2:
                    rect.setCoords(px, py + height, px - height, py)
                    rect = rect.normalized()

                elif self._resizingFrom == 3:
                    rect.setCoords(px, py, px + height, py + height)
                    rect = rect.normalized()

                elif self._resizingFrom == 4:
                    rect.setCoords(px - height, py, px, py + height)
                    rect = rect.normalized()

                # Alt Key - to resize around center
                if event.modifiers() == Qt.KeyboardModifier.AltModifier:
                    rect.moveCenter(c)

            fark = event.pos() - self._ilk_tik_event_pos

            # self.setRect(rect)
            # self.setPos(newPos)
            # TODO : alt calismiyor move to 0,0 dan dolayi
            self._resize(rect, fark)  # mouse release eventten gonderiyoruz undoya
            # self.setRect(rect)  # mouse release eventten gonderiyoruz undoya
            self.update_resize_handles()
            self.scene().parent().change_transform_box_values(self)
            # self.setPos(x, y)
            # return QGraphicsItem.mouseMoveEvent(self, event)

            # self.update_painter_text_rect()

        # event.accept()
        else:
            super(Group, self).mouseMoveEvent(event)

        # super(Group, self).mouseMoveEvent(event)

    # ---------------------------------------------------------------------
    def mouseReleaseEvent(self, event):
        if self._resizing:

            yeniRect = self._rect

            try:
                scaleFactorX = yeniRect.width() / self._eskiRectBeforeResize.width()
                scaleFactorY = yeniRect.height() / self._eskiRectBeforeResize.height()
            except ZeroDivisionError:
                # scaleFactorX = scaleFactorY = 1
                super(Group, self).mouseReleaseEvent(event)
                return

            # fark = yeniRect.topLeft() - self._eskiRectBeforeResize.topLeft()

            yeniPos = self.mapToScene(yeniRect.topLeft())
            if self.parentItem():
                yeniPos = self.mapToParent(yeniRect.topLeft())
            # self.setPos(yeniPos)
            # yeniRect.moveTo(0, 0)
            # self.update_resize_handles()

            # self.setPos(self.mapToScene(yeniRect.topLeft()))

            self.scene().undoRedo.undoableResizeGroupItem(self.scene().undoStack,
                                                          "_resized by group",
                                                          self,
                                                          self._eskiRectBeforeResize,
                                                          yeniRect,
                                                          self._eskiPosBeforeResize,
                                                          yeniPos,
                                                          scaleFactorX,
                                                          scaleFactorY)

            self._resizing = False

            # self.update_painter_text_rect()
            self.scene().unite_with_scene_rect(self.sceneBoundingRect())
        super(Group, self).mouseReleaseEvent(event)

        if self.parentedWithParentOperation:
            if self.isSelected():
                self.scene().select_all_children_recursively(self, cosmeticSelect=False, topmostParent=True)
            else:
                self.scene().select_all_children_recursively(self, cosmeticSelect=False, topmostParent=False)

    # ---------------------------------------------------------------------
    def _resize(self, yeniRect, fark):
        if self._rect == yeniRect:
            return

        # eskiRect = self._rect

        # try:
        #     scaleFactorX = yeniRect.width() / eskiRect.width()
        #     scaleFactorY = yeniRect.height() / eskiRect.height()
        # except ZeroDivisionError:
        #     # scaleFactorX = scaleFactorY = 1
        #     return

        # yeniPos = self.mapToScene(yeniRect.topLeft())
        # if self.parentItem():
        #     yeniPos = self.mapToParent(yeniRect.topLeft())
        # yeniPos = yeniPos + fark
        # self.setPos(yeniPos)
        # yeniRect.moveTo(0,0)
        # for c in self.allFirstLevelGroupChildren:
        #     # c.setPos(c.pos()+fark)
        #     c.moveBy(fark.x(), fark.y())

        self.setRect(yeniRect)

        # shared._scaleChildItemsByResizing(self, fark, scaleFactorX, scaleFactorY)

    # ---------------------------------------------------------------------
    def type(self):
        return Group.Type

    # ---------------------------------------------------------------------
    @property
    def isPinned(self):
        return self._isPinned

    # ---------------------------------------------------------------------
    @isPinned.setter
    def isPinned(self, value):
        if value:
            self.setFlags(QGraphicsItem.GraphicsItemFlag.ItemIsSelectable
                          # | QGraphicsItem.ItemIsMovable
                          #  | QGraphicsItem.ItemIsFocusable
                          )

        else:
            self.setFlags(QGraphicsItem.GraphicsItemFlag.ItemIsSelectable
                          | QGraphicsItem.GraphicsItemFlag.ItemIsMovable
                          | QGraphicsItem.GraphicsItemFlag.ItemIsFocusable)
        self._isPinned = value

    # ---------------------------------------------------------------------
    def ok_ekle(self, ok, scenepPos, okunHangiNoktasi):

        # self.oklar_dxdy_nokta.append((ok, self.mapFromScene(scenePos)))
        sceneCenter = self.sceneCenter()
        dx = sceneCenter.x() - scenepPos.x()
        dy = sceneCenter.y() - scenepPos.y()

        # self.oklar_dxdy_nokta[ok._kim] = (dx, dy, nokta)
        self.oklar_dxdy_nokta[ok] = (dx, dy, okunHangiNoktasi)

        ok.baglanmis_nesneler[self._kim] = okunHangiNoktasi

    # ---------------------------------------------------------------------
    def ok_sil(self, ok):
        del self.oklar_dxdy_nokta[ok]
        del ok.baglanmis_nesneler[self._kim]

    # ---------------------------------------------------------------------
    def ok_guncelle(self):
        if self.oklar_dxdy_nokta:
            sceneCenter = self.sceneCenter()
            scx = sceneCenter.x()
            scy = sceneCenter.y()
            for ok, dxdy_nokta in self.oklar_dxdy_nokta.items():
                if dxdy_nokta[2] == 1:
                    ok.temp_prepend(QPointF(scx - dxdy_nokta[0], scy - dxdy_nokta[1]))
                elif dxdy_nokta[2] == 2:
                    ok.temp_append(QPointF(scx - dxdy_nokta[0], scy - dxdy_nokta[1]))
                if ok._text:
                    ok.update_painter_text_rect()

    # ---------------------------------------------------------------------
    def varsaEnUsttekiGrubuGetir(self):
        parentItem = self.parentItem()
        while parentItem:
            if parentItem.type() == shared.GROUP_ITEM_TYPE:
                return parentItem
            parentItem = parentItem.parentItem()
        return None

    # ---------------------------------------------------------------------
    def get_properties_for_save_binary(self):
        properties = {"type": self.type(),
                      "kim": self._kim,
                      "pos": self.pos(),
                      "rotation": self.rotation(),
                      "zValue": self.zValue(),
                      "yaziRengi": self.yaziRengi,  # yaziRengi grup nesnesinde aktik olarak kullanilmiyor
                      "arkaPlanRengi": self.arkaPlanRengi,  # arkaPlanRengi grup nesnesinde aktik olarak kullanilmiyor
                      "pen": self._pen,
                      "isPinned": self.isPinned,
                      }
        return properties

    # ---------------------------------------------------------------------
    def contextMenuEvent(self, event):
        # TODO: bu alttaki iki satir aktif idi, ve de ctrl harici sag tiklayinca
        # (base- text -path -group nesnelerinin contextMenuEventinde var)
        # mesela birden fazla nesne secili ve de gruplayacagız diyelim sag menu ile
        # ctrl basılı degil ise tikladigimiz secili kaliyor digerleri siliniyordu
        # uygunsuz bir kullanıcı deneyimi, niye yaptık acaba boyleyi hatırlayana kadar kalsın burda :)
        # if not event.modifiers() == Qt.KeyboardModifier.ControlModifier:
        #     self.scene().clearSelection()
        self.setSelected(True)

        # self.scene().parent().group_item_context_menu(self, event.screenPos())
        self.scene().parent().on_grup_sag_menu_about_to_show(self)
        self.scene().parent().grupSagMenu.popup(event.screenPos())
        event.accept()

    # ---------------------------------------------------------------------
    def sceneCenter(self):
        if self.parentItem():
            return self.mapToParent(self.boundingRect().center())
        return self.mapToScene(self.boundingRect().center())

    # ---------------------------------------------------------------------
    def sceneWidth(self):
        return self.sceneRight() - self.sceneLeft()

    # ---------------------------------------------------------------------
    def sceneHeight(self):
        return self.sceneBottom() - self.sceneTop()

    # ---------------------------------------------------------------------
    def sceneRight(self):
        return max(self.mapToScene(self.boundingRect().topRight()).x(),
                   self.mapToScene(self.boundingRect().bottomRight()).x())

    # ---------------------------------------------------------------------
    def sceneLeft(self):
        return min(self.mapToScene(self.boundingRect().topLeft()).x(),
                   self.mapToScene(self.boundingRect().bottomLeft()).x())

    # ---------------------------------------------------------------------
    def sceneTop(self):
        return min(self.mapToScene(self.boundingRect().topLeft()).y(),
                   self.mapToScene(self.boundingRect().topRight()).y())

    # ---------------------------------------------------------------------
    def sceneBottom(self):
        return max(self.mapToScene(self.boundingRect().bottomRight()).y(),
                   self.mapToScene(self.boundingRect().bottomLeft()).y())

    # ---------------------------------------------------------------------
    def setSceneLeft(self, left):
        self.setX(left + self.scenePos().x() - self.sceneLeft())

    # ---------------------------------------------------------------------
    def setSceneRight(self, right):
        self.setX(right + self.scenePos().x() - self.sceneRight())

    # ---------------------------------------------------------------------
    def setSceneTop(self, top):
        self.setY(top + self.scenePos().y() - self.sceneTop())

    # ---------------------------------------------------------------------
    def setSceneBottom(self, bottom):
        self.setY(bottom + self.scenePos().y() - self.sceneBottom())

    # ---------------------------------------------------------------------
    def setHorizontalCenter(self, hcenter):
        self.setY(hcenter + self.scenePos().y() - self.sceneCenter().y())

    # ---------------------------------------------------------------------
    def setVerticalCenter(self, vcenter):
        self.setX(vcenter + self.scenePos().x() - self.sceneCenter().x())

    # ---------------------------------------------------------------------
    def moveCenterToPosition(self, scenePos):
        self.setPos(scenePos - self._rect.center())

    # ---------------------------------------------------------------------
    def flipHorizontal(self, mposx):

        eskiRot = self.rotation()
        if eskiRot:
            self.rotateWithOffset(0)

        for c in self.allFirstLevelGroupChildren:
            c.flipHorizontal(mposx)
        self.updateBoundingRect()

        # burda, yani grupta, base itema gore biraz daha degisik bir durum var,
        # zaten burda basta sifirliyoruz rotationu, tekrar sifirlamaya gerek kalmiyor
        # eger grup item ayni zamanda normal parent ise, icerigi asagida flip ediyoruz.
        for c in self.parentedWithParentOperation:
            c.flipHorizontal(mposx)

        if eskiRot:
            self.rotateWithOffset(360 - eskiRot)

    # ---------------------------------------------------------------------
    def flipVertical(self, mposy):

        eskiRot = self.rotation()
        if eskiRot:
            self.rotateWithOffset(0)

        for c in self.allFirstLevelGroupChildren:
            c.flipVertical(mposy)
        self.updateBoundingRect()

        for c in self.parentedWithParentOperation:
            c.flipVertical(mposy)

        if eskiRot:
            self.rotateWithOffset(360 - eskiRot)

    # ---------------------------------------------------------------------
    def _update_rect_recursively(self, items, rect):

        for c in items:
            # rect = rect.united(self.mapRectFromItem(c, c.boundingRect()))
            rect |= self.mapRectFromItem(c, c.boundingRect())
            # burda sub item grup olsa bile ayirmaya gerek yok "allFirstLevelGroupChildren" diye childitemlari
            # cunku asıl grup hepsini kapsamak zorunda
            if c.childItems():
                rect = self._update_rect_recursively(c.childItems(), rect)

        return rect

    # ---------------------------------------------------------------------
    def updateBoundingRect(self):
        self.prepareGeometryChange()  # TODO: bunu debug et acaip uzun suruyor
        # rect = self.childrenBoundingRect()

        rect = QRectF()
        # for item in self.childItems():
        for item in self.allFirstLevelGroupChildren:
            # rect = rect.united(self.mapRectFromItem(item, item.boundingRect()))
            rect |= self.mapRectFromItem(item, item.boundingRect())
            # burda sub item grup olsa bile ayirmaya gerek yok "allFirstLevelGroupChildren" diye childitemlari
            # cunku asıl grup hepsini kapsamak zorunda
            if item.childItems():
                rect = self._update_rect_recursively(item.childItems(), rect)

        # TODO: bir bak
        # pozisyonu sahne koordinatlarında topLefte almak icin,
        # ama scaled group mirrorunu bozuyor.
        diff = self.scenePos() - self.mapToScene(rect.topLeft())
        self.setPos(self.mapToScene(rect.topLeft()))
        rect.moveTo(0, 0)
        self.setRect(rect)
        for c in self.allFirstLevelGroupChildren:
            # c.setPos(c.pos()+diff)
            c.moveBy(diff.x(), diff.y())

        # self.tumElemanlariEkleyincePozisyonGuncelle()

    # # ---------------------------------------------------------------------
    # def tumElemanlariEkleyincePozisyonGuncelle(self):
    #     # self.updateBoundingRect()
    #     self.setPos(self.mapToScene(self._rect.topLeft()))

    # ---------------------------------------------------------------------
    def addSingleItemToGroup(self, item):
        if not item:
            return
        # self.prepareGeometryChange()
        eskiItemRot = item.rotation()
        item.setParentItem(self)
        item.ustGrup = self
        item.rotateWithOffset(eskiItemRot - self.rotation())
        item.isPinned = True  # !! bunun yeri onemli , asagidaki flagleri set edisimizin altında bunu set edersek,
        # isPinned bir property oldugundan, set kisminda her halukarda  "ItemIsSelectable ,True"oldugundan override
        # ediyor ve gruplanmis item secilebilir oluyor.
        item.setFlag(QGraphicsItem.GraphicsItemFlag.ItemIsSelectable, False)
        item.setFlag(QGraphicsItem.GraphicsItemFlag.ItemIsMovable, False)
        item.setFlag(QGraphicsItem.GraphicsItemFlag.ItemIsFocusable, False)
        item.setFlag(QGraphicsItem.GraphicsItemFlag.ItemStacksBehindParent, True)
        # TODO: ispinned e niye gerek duyduk.. destroy groupta da false ediliyor

        if item.type() == self.type():
            # TODO: set ve daha sonra destroy mu kullansak, ungroup ederken.
            self.allNonGroupGroupChildren.extend(item.allNonGroupGroupChildren)
        else:
            self.allNonGroupGroupChildren.append(item)
            if item.childItems():
                self.allNonGroupGroupChildren.extend(item.childItems())
        # we re adding directly because, if an item and its parent is selected, we don't add item to a group,
        # we re adding only its parent.
        self.allFirstLevelGroupChildren.append(item)

    # ---------------------------------------------------------------------
    def destroyGroupForPaste(self):
        # python deletes these "reparented to None" items, if we dont keep references to them.
        unGroupedRootItems = []

        # for item in self.childItems():
        # + self.parentedWithParentOperation cunku normal parent edilmis nesneleri ungroup edince siliyor.
        for item in self.allFirstLevelGroupChildren:  # + self.parentedWithParentOperation:
            # if for first level child items
            # if item.parentItem() == self:
            pos = self.mapToScene(item.pos())
            unGroupedRootItems.append(item)
            item.setParentItem(None)
            item.setPos(pos)
            item.setSelected(True)

        self.scene().unGroupedRootItems.update(unGroupedRootItems)

    # ---------------------------------------------------------------------
    def destroyGroup(self):
        # items = self.childItems()

        # python deletes these "reparented to None" items, if we dont keep references to them.
        unGroupedRootItems = []

        # for item in self.childItems():
        # + self.parentedWithParentOperation cunku normal parent edilmis nesneleri ungroup edince siliyor.
        # self.parentedWithParentOperation[:] kopyasi cunku : itemchange te otomatik olarak bosaltildigi icin
        for item in self.allFirstLevelGroupChildren + self.parentedWithParentOperation[:]:
            # if for first level child items
            # if item.parentItem() == self:
            parent = self.parentItem()
            if not parent:
                pos = self.mapToScene(item.pos())
                unGroupedRootItems.append(item)
                item.setParentItem(parent)  # sets parent to None
            else:
                pos = self.mapToItem(parent, item.pos())  # eger grup parented ise, cozulunce icerigi ayni yerde kalsin.
                item.setParentItem(parent)
                if parent.type() == self.type():
                    parent.parentedWithParentOperation.append(item)

            item.isPinned = False
            item.ustGrup = None
            if not item.type() == shared.LINE_ITEM_TYPE:
                item.rotateWithOffset(self.rotation() + item.rotation())
            else:
                item.rotateWithOffset(self.rotation() - item.rotation())
                # item._line.setAngle(item._line.angle() - self.rotation())
                # item.update_arrow()
            item.setPos(pos)
            item.setFlags(QGraphicsItem.GraphicsItemFlag.ItemIsSelectable |
                          QGraphicsItem.GraphicsItemFlag.ItemIsMovable |
                          QGraphicsItem.GraphicsItemFlag.ItemIsFocusable)
            item.setFlag(QGraphicsItem.GraphicsItemFlag.ItemStacksBehindParent, False)
            # TODO: info: gruplanmiş parentlarin childlerini grupta tutmuyoruz. dolayisi ile
            # burda manual ayarliyoruz flaglari. yoksa secilemez oluyrolar. manual false etmistik zaten act_groupta.
            if not item.type() == self.type():  # grubun iceriginin secilemezligi korunsun diye
                for c in item.childItems():
                    c.setFlags(QGraphicsItem.GraphicsItemFlag.ItemIsSelectable |
                               QGraphicsItem.GraphicsItemFlag.ItemIsMovable |
                               QGraphicsItem.GraphicsItemFlag.ItemIsFocusable)
                    c.setFlag(QGraphicsItem.GraphicsItemFlag.ItemStacksBehindParent, False)
                    c.setSelected(True)
            else:
                for c in item.parentedWithParentOperation:
                    c.setFlags(QGraphicsItem.GraphicsItemFlag.ItemIsSelectable |
                               QGraphicsItem.GraphicsItemFlag.ItemIsMovable |
                               QGraphicsItem.GraphicsItemFlag.ItemIsFocusable)
                    c.setFlag(QGraphicsItem.GraphicsItemFlag.ItemStacksBehindParent, False)
                    c.setSelected(True)
            item.setSelected(True)
        self.scene().unGroupedRootItems.update(unGroupedRootItems)
        self.setSelected(False)  # to remove from selectionQueue
        self.parentedWithParentOperation = []
        self.allFirstLevelGroupChildren = []
        self.allNonGroupGroupChildren = []

        # no need to below, because we will delete this instance immediately.
        # self.prepareGeometryChange()
        # self.setRect(self.childrenBoundingRect())

    # ---------------------------------------------------------------------
    def boundingRect(self):
        boundingRect = QRectF(self._rect)
        return boundingRect.adjusted(-self.boundingRectTasmaDegeri, -self.boundingRectTasmaDegeri,
                                     self.boundingRectTasmaDegeri, self.boundingRectTasmaDegeri)
        # return self._rect

    # # ---------------------------------------------------------------------
    # def shape(self):
    #     path = QPainterPath()
    #     # path.addRect(self.rect)
    #     path.addRect(self._rect)
    #     # return self.qt_graphicsItem_shapeFromPath(path, self._pen)
    #     return path

    # ---------------------------------------------------------------------
    def itemChange(self, change, value):
        # if change == self.ItemPositionChange:
        # if change == self.ItemSelectedChange:
        # if change == self.ItemSelectedHasChanged:
        if change == QGraphicsItem.GraphicsItemChange.ItemSelectedChange:
            if value:
                self.scene().parent().group_item_selected(self)
                # self.selectAllChildren()
            else:
                self.scene().parent().group_item_deselected(self)
                # self.deSelectAllChildren()

        if change == QGraphicsItem.GraphicsItemChange.ItemChildRemovedChange:  # sahneden delete ile mesela remove edince, bir de unparent edince
            # TODO: allNonGroupGroupChildren yerine bu changed kullanilabilir mi acaba.
            if value in self.parentedWithParentOperation:
                self.parentedWithParentOperation.remove(value)

        # return QGraphicsRectItem.itemChange(self, change, value)
        return value
        # super(Rect, self).itemChange(change, value)

    # ---------------------------------------------------------------------
    def setPen(self, pen):

        if self._pen == pen:
            return
        self.prepareGeometryChange()
        self._pen = pen
        # self._boundingRect = QRectF()
        self.update(self.boundingRect())

    # ---------------------------------------------------------------------
    def pen(self):
        return self._pen

    # ---------------------------------------------------------------------
    def setCizgiRengi(self, col):

        _selectionLineBgColor = QColor.fromHsv(col.hue(),
                                               0,
                                               20 if col.value() > 127 else 250)

        _activeItemLineColor = QColor(self.activeItemLineColor)
        if col.hue() > 300 or col.hue() < 30:
            _activeItemLineColor.setHsv((col.hue() + 150) % 360,
                                        self.activeItemLineColor.saturation(),
                                        self.activeItemLineColor.value())
        self._pen.setColor(col)
        self.cizgiRengi = col

        self.selectionPenBottom = QPen(_selectionLineBgColor,
                                       self.secili_nesne_kalem_kalinligi,
                                       Qt.PenStyle.DashLine,
                                       Qt.PenCapStyle.RoundCap, Qt.PenJoinStyle.RoundJoin)
        self.selectionPenBottomIfAlsoActiveItem = QPen(_activeItemLineColor,
                                                       self.secili_nesne_kalem_kalinligi,
                                                       Qt.PenStyle.DashLine,
                                                       Qt.PenCapStyle.RoundCap, Qt.PenJoinStyle.RoundJoin)

        if col.value() > 245:
            self.handleBrush = shared.handleBrushKoyu
        else:
            self.handleBrush = shared.handleBrushAcik

        self.update()

    # ---------------------------------------------------------------------
    def setCizgiKalinligi(self, width):
        self._pen.setWidthF(width)
        self.boundingRectTasmaDegeri = ((self._pen.widthF() / 2) + (self.handleSize / 2))
        # self.textPen.setWidthF(width)
        # self.selectionPenBottom.setWidthF(width)
        # self.selectionPenBottomIfAlsoActiveItem.setWidthF(width)
        # self.selectionPenTop.setWidthF(width)

        self.update()

    # ---------------------------------------------------------------------
    def setArkaPlanRengi(self, col):
        # arkaplanRengi ve yaziRengi grup nesnesinde kullanilmiyor
        # fasulyeden method
        self.arkaPlanRengi = col

    # ---------------------------------------------------------------------
    def setYaziRengi(self, col):
        # arkaplanRengi ve yaziRengi grup nesnesinde kullanilmiyor
        # fasulyeden method
        self.yaziRengi = col

    # ---------------------------------------------------------------------
    def hoverEnterEvent(self, event):
        # event propagationdan dolayi bos da olsa burda implement etmek lazim
        # mousePress,release,move,doubleclick de bu mantıkta...
        # baska yerden baslayip mousepress ile , mousemove baska, mouseReleas baska widgetta gibi
        # icinden cikilmaz durumlara sebep olabiliyor.
        # ayrica override edince accept() de edilmis oluyor mouseeventler
        super(Group, self).hoverEnterEvent(event)

    # ---------------------------------------------------------------------
    def hoverMoveEvent(self, event):

        # cursor = self.scene().parent().cursor()

        # if self.isSelected() and self.scene().aktifArac == self.scene().SecimAraci:
        if not self.ustGrup:
            if self.scene().aktifArac == self.scene().SecimAraci:
                if self.topLeftHandle.contains(event.pos()) or self.bottomRightHandle.contains(event.pos()):
                    self.scene().parent().setCursor(Qt.CursorShape.SizeFDiagCursor, gecici_mi=True)
                    # self.setCursor(Qt.SizeFDiagCursor, gecici_mi=True)
                elif self.topRightHandle.contains(event.pos()) or self.bottomLeftHandle.contains(event.pos()):
                    self.scene().parent().setCursor(Qt.CursorShape.SizeBDiagCursor, gecici_mi=True)
                    # self.setCursor(Qt.SizeBDiagCursor, gecici_mi=True)
                else:
                    self.scene().parent().setCursor(self.scene().parent().imlec_arac, gecici_mi=True)

        super(Group, self).hoverMoveEvent(event)

    # ---------------------------------------------------------------------
    def hoverLeaveEvent(self, event):
        # if self.isSelected():
        # self.setCursor(self.saved_cursor)
        if not self.ustGrup:
            self.scene().parent().setCursor(self.scene().parent().imlec_arac, gecici_mi=True)

        super(Group, self).hoverLeaveEvent(event)

    # ---------------------------------------------------------------------
    def wheelEvent(self, event):

        if self.ustGrup:
            return QGraphicsItem.wheelEvent(self, event)

        toplam = event.modifiers().value

        # ctrl = int(Qt.ControlModifier)
        shift = Qt.KeyboardModifier.ShiftModifier.value
        alt = Qt.KeyboardModifier.AltModifier.value

        ctrlAlt = Qt.KeyboardModifier.ControlModifier.value + Qt.KeyboardModifier.AltModifier.value
        ctrlShift = Qt.KeyboardModifier.ControlModifier.value + Qt.KeyboardModifier.ShiftModifier.value
        altShift = Qt.KeyboardModifier.AltModifier.value + Qt.KeyboardModifier.ShiftModifier.value
        ctrlAltShift = Qt.KeyboardModifier.ControlModifier.value + Qt.KeyboardModifier.AltModifier.value + Qt.KeyboardModifier.ShiftModifier.value

        # if event.modifiers() & Qt.ControlModifier:

        if toplam == ctrlShift:
            # self.scaleItem(event.delta())
            self.scaleItemByResizing(event.delta())

        # elif event.modifiers() & Qt.ShiftModifier:
        elif toplam == shift:
            self.changeFontSizeF(event.delta())

        elif toplam == alt:
            self.changeBackgroundColorAlpha(event.delta())

        elif toplam == ctrlAlt:
            self.changeLineColorAlpha(event.delta())

        elif toplam == ctrlAltShift:
            self.rotateItem(event.delta())

        elif toplam == altShift:
            self.changeImageItemTextBackgroundColorAlpha(event.delta())
        #
        # elif toplam == ctrlAltShift:
        #     pass

        else:
            super(Group, self).wheelEvent(event)

    # ---------------------------------------------------------------------
    def scaleItemByResizing(self, delta):

        scaleFactor = 1.1
        if delta < 0:
            scaleFactor = 1 / scaleFactor

        # eskiTamCerceve = self.boundingRect() | self.childrenBoundingRect()
        # rect = self.childrenBoundingRect()
        rect = self._rect
        center = rect.center()

        rect = QRectF(rect.topLeft(), rect.size() * scaleFactor)
        rect.moveCenter(center)
        yeniPos = self.mapToScene(rect.topLeft())
        if self.parentItem():
            yeniPos = self.mapToParent(rect.topLeft())
        rect.moveTo(0, 0)

        self.scene().undoRedo.undoableScaleGroupItemByResizing(self.scene().undoStack,
                                                               "_scale by resizing",
                                                               self, rect, scaleFactor,
                                                               yeniPos)

        # self.updateBoundingRect()

    # ---------------------------------------------------------------------
    def rotateWithOffset(self, angle):
        cEski = self.sceneCenter()
        self.setRotation(angle)
        cYeni = self.sceneCenter()
        diff = cEski - cYeni
        self.moveBy(diff.x(), diff.y())
        # self.updateBoundingRect()

    # ---------------------------------------------------------------------
    def rotateItem(self, delta):
        if delta > 0:
            # self.setRotation(self.rotation() + 5)
            # self.rotateWithOffset(self.rotation() + 5)
            self.scene().undoRedo.undoableRotateWithOffset(self.scene().undoStack, "rotate", self, self.rotation() + 5)
        else:
            # self.setRotation(self.rotation() - 5)
            # self.rotateWithOffset(self.rotation() - 5)
            self.scene().undoRedo.undoableRotateWithOffset(self.scene().undoStack, "rotate", self, self.rotation() - 5)

    # ---------------------------------------------------------------------
    def changeBackgroundColorAlpha(self, delta):
        self.scene().undoStack.beginMacro("change items' background alpha")
        for c in self.childItems():
            c.changeBackgroundColorAlpha(delta)
        self.scene().undoStack.endMacro()

    # ---------------------------------------------------------------------
    def changeLineColorAlpha(self, delta):
        self.scene().undoStack.beginMacro("change items' line alpha")
        for c in self.childItems():
            c.changeLineColorAlpha(delta)
        self.scene().undoStack.endMacro()

    # ---------------------------------------------------------------------
    def changeFontSizeF(self, delta):
        self.scene().undoStack.beginMacro("change text size")
        for c in self.childItems():
            c.changeFontSizeF(delta)
        self.scene().undoStack.endMacro()

    # ---------------------------------------------------------------------
    def changeImageItemTextBackgroundColorAlpha(self, delta):
        self.scene().undoStack.beginMacro("change image items' text background alpha")
        for c in self.childItems():
            c.changeImageItemTextBackgroundColorAlpha(delta)
        self.scene().undoStack.endMacro()

    # ---------------------------------------------------------------------
    def paint(self, painter, option, widget=None):
        # painter.setBrush(self.arkaPlanRengi)

        painter.setPen(self._pen)
        # painter.drawRect(self.boundingRect())
        painter.drawRect(self._rect)

        if option.state & QStyle.StateFlag.State_Selected or self.cosmeticSelect:
            if self.isActiveItem:
                selectionPenBottom = self.selectionPenBottomIfAlsoActiveItem
                if not self.isPinned:
                    painter.setPen(Qt.PenStyle.NoPen)
                    painter.setBrush(self.handleBrush)
                    painter.drawRect(self.topLeftHandle)
                    painter.drawRect(self.topRightHandle)
                    painter.drawRect(self.bottomRightHandle)
                    painter.drawRect(self.bottomLeftHandle)
            else:
                selectionPenBottom = self.selectionPenBottom

            painter.setBrush(Qt.BrushStyle.NoBrush)
            painter.setPen(selectionPenBottom)
            # painter.drawRect(self.boundingRect())
            painter.drawRect(self._rect)

            # painter.setPen(self.selectionPenTop)
            # painter.drawRect(self.boundingRect())

            #######################################################################
            # !!! simdilik iptal, gorsel fazlalik olusturmakta !!!
            #######################################################################
            # if not self.isPinned and self.isActiveItem:
            # painter.setPen(self.handlePen)
            # painter.setPen(Qt.PenStyle.NoPen)
            # painter.setBrush(self.handleBrush)
            # painter.drawRect(self.topLeftHandle)
            # painter.drawRect(self.topRightHandle)
            # painter.drawRect(self.bottomRightHandle)
            # painter.drawRect(self.bottomLeftHandle)

            # painter.setPen(QPen(Qt.GlobalColor.red, 3))
            # painter.drawPoint(self.topLeftHandle.center())
            # painter.drawPoint(self.topRightHandle.center())
            # painter.drawPoint(self.bottomRightHandle.center())
            # painter.drawPoint(self.bottomLeftHandle.center())

        # if option.state & QStyle.StateFlag.State_MouseOver:
        #     painter.setBrush(Qt.BrushStyle.NoBrush)
        #     painter.setPen(self.selectionPenBottom)
        #     painter.drawRect(self._rect)

        # # # # # debug start - pos() # # # # #
        # p = self.pos()
        # s = self.scenePos()
        # painter.drawText(self.boundingRect(),
        #                  "{0:.2f},  {1:.2f} pos \n{2:.2f},  {3:.2f} spos".format(p.x(), p.y(), s.x(), s.y()))
        # painter.setPen(QPen(Qt.green, 20))
        # painter.drawPoint(0, 0)
        # painter.drawPoint(self.mapFromScene(self.sceneBoundingRect().center()))
        # painter.setPen(QPen(Qt.cyan, 24))
        # painter.drawPoint(p)
        # painter.setPen(QPen(Qt.yellow, 20))
        # painter.drawPoint(s)
        # painter.setPen(QPen(Qt.red, 16))
        # painter.drawPoint(self.mapFromScene(p))
        # painter.setPen(QPen(Qt.blue, 12))
        # painter.drawPoint(self.mapFromScene(s))
        # t = self.transformOriginPoint()
        # painter.drawRect(t.x()-12, t.y()-12,24,24)
        # mapped = self.mapToScene(self.boundingRect().topLeft())
        # painter.drawText(self.boundingRect().center().x(), self.boundingRect().center().y(),
        #                  "{0:.2f}  {1:.2f}".format(mapped.x(), mapped.y()))
        # painter.drawText(self.boundingRect().center().x(), self.boundingRect().center().y(),
        #                  "{0:.3f}".format(self.scale()))
        # painter.drawRect(self.boundingRect())
        # painter.setPen(QPen(Qt.yellow, 1))
        # painter.drawRect(QRectF(0, 0, 50, 50))
        # painter.drawText(self._rect.center(), "{0:f}  {1:f}".format(self.sceneWidth(), self.sceneHeight()))
        # # # # # debug end - pos() # # # # #

    # ---------------------------------------------------------------------
    def html_dive_cevir(self, html_klasor_kayit_adres, dosya_kopyalaniyor_mu):
        return ""



# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__author__ = 'Erdinç Yılmaz'
__date__ = '3/28/16'

from PySide6.QtWidgets import QDialog, QVBoxLayout, QHBoxLayout, QTextEdit, QPushButton, QLineEdit, QApplication
from PySide6.QtCore import Qt, Slot, QSettings
from . import shared


########################################################################
class CommandDialog(QDialog):

    # logger = Signal(str)

    # def __init__(self, item, blenderHistory=None, parent=None):
    def __init__(self, item, parent=None):
        super(CommandDialog, self).__init__(parent)

        self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose)
        self.setWindowFlags(Qt.WindowType.WindowStaysOnTopHint)
        layout = QVBoxLayout(self)
        btnLayout = QHBoxLayout()

        self.lineEdit = QLineEdit(self)
        self.lineEdit.setPlaceholderText("Title")
        layout.addWidget(self.lineEdit)
        self.textEdit = QTextEdit(self)
        layout.addWidget(self.textEdit)
        command = item.command()
        if command:
            self.lineEdit.setText(command["title"])
            self.textEdit.setPlainText(command["command"])

        # blenderHistoryListWidget = QListWidget(self)
        # editlayout.addWidget(blenderHistoryListWidget)
        closeBtn = QPushButton("&Close", self)
        closeBtn.setDefault(True)
        closeBtn.clicked.connect(self.reject)
        saveBtn = QPushButton("&Save", self)
        saveBtn.clicked.connect(lambda: self.act_save_command(item))
        btnLayout.addStretch()
        btnLayout.addWidget(closeBtn)
        btnLayout.addWidget(saveBtn)
        layout.addLayout(btnLayout)

        self.olustur_ayarlar()
        self.oku_ayarlar()
        # commandDialog.accepted.connect()

    # ---------------------------------------------------------------------
    def closeEvent(self, event):
        self.yaz_ayarlar()
        event.accept()

    # ---------------------------------------------------------------------
    def olustur_ayarlar(self):
        QApplication.setOrganizationName(shared.DEFTER_ORG_NAME)
        QApplication.setOrganizationDomain(shared.DEFTER_ORG_DOMAIN)
        QApplication.setApplicationName(shared.DEFTER_APP_NAME)
        self.settings = QSettings(shared.DEFTER_AYARLAR_DOSYA_ADRES, QSettings.Format.IniFormat)
        # self.settings.clear()

    # ---------------------------------------------------------------------
    def oku_ayarlar(self):
        self.settings.beginGroup("CommandDialogSettings")
        if self.settings.contains("CWinGeometry"):
            self.restoreGeometry(self.settings.value("CWinGeometry"))
        self.settings.endGroup()

    # ---------------------------------------------------------------------
    def yaz_ayarlar(self):
        self.settings.beginGroup("CommandDialogSettings")
        self.settings.setValue("CWinGeometry", self.saveGeometry())
        self.settings.endGroup()

    # ---------------------------------------------------------------------
    @Slot()
    def act_save_command(self, item):
        title = self.lineEdit.text()
        command = self.textEdit.toPlainText()
        if command:
            item.setCommand(title=title,
                            command=command)
            item.setText(title)
        # else:
        #     self.logger.emit("no command")



# -*- coding: utf-8 -*-

__project_name__ = 'defter'
__date__ = '10/6/23'
__author__ = 'E. Y.'

from PySide6.QtWidgets import QToolBar


#######################################################################
class ToolBar(QToolBar):

    def __init__(self, title, parent=None):
        super(ToolBar, self).__init__(title, parent)

        stil2 = """

         QToolBar {
                background: #eee;
                padding: 0px;
                border: 0px;
                border-radius: 5px;
            }
        QToolBar QToolButton{
            background-color: #eee;
            border: 0px;
            padding: 5px;
            border-radius: 5px;
        }
                QToolBar QToolButton:hover{
            background-color: #def;
        }
        QToolBarExtension {
            background-color: black;
        }
        QPushButton QMenu
        {
            height: 1px;
            border-bottom: 1px solid lightGray;
            background: #5A5A5A;
            margin-left: 2px;
            margin-right: 0px;
            margin-top: 2px;
            margin-bottom: 2px;
         }
        QPushButton QMenu::separator
        {
            height: 1px;
            border-bottom: 1px solid lightGray;
            background: #5A5A5A;
            margin-left: 2px;
            margin-right: 0px;
            margin-top: 2px;
            margin-bottom: 2px;
         }
        
         QToolButton { /* all types of tool button */
            border: 2px solid #8f8f91;
            border-radius: 6px;
            background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1,
                                              stop: 0 #f6f7fa, stop: 1 #dadbde);
        }
        
        QToolButton[popupMode="1"] { /* only for MenuButtonPopup */
            padding-right: 20px; /* make way for the popup button */
        }
        
        QToolButton:pressed {
            background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1,
                                              stop: 0 #dadbde, stop: 1 #f6f7fa);
        }
        
        /* the subcontrols below are used only in the MenuButtonPopup mode */
        QToolButton::menu-button {
            border: 2px solid gray;
            border-top-right-radius: 6px;
            border-bottom-right-radius: 6px;
            /* 16px width + 4px for border = 20px allocated above */
            width: 16px;
        }
        
        QToolButton::menu-arrow {
            image: url(downarrow.png);
        }
        
        QToolButton::menu-arrow:open {
            top: 1px; left: 1px; /* shift it a bit */
        }

                """

        # border-radius: 5px;

        stil = """
         QToolBar {
                background: #eee;
                padding: 3px;
                border: 0px;
                border-radius: 3px;

            }
        QToolBar QToolButton{
            background-color: #eee;
            border: 0;
            padding: 3px;
            border-radius: 3px;
        }
        QToolBar QToolButton:hover{
            background-color: #def;
        }
        QToolBar QToolButton:pressed{
            background-color: #cde;
        }
        QToolBar QToolButton:checked{
            background-color: #ced5e1;
        }
        QToolButton[popupMode="1"] { /* only for MenuButtonPopup */
            padding-right: 20px; /* make way for the popup button */
        }
        
        /* the subcontrols below are used only in the MenuButtonPopup mode */
        QToolButton::menu-button {
            border: 0px;
            border-top-right-radius: 5px;
            border-bottom-right-radius: 5px;
            /* 16px width + 4px for border = 20px allocated above */
            width: 16px;
        }


        QToolButton::menu-arrow:open {
            top: 1px; left: 1px; /* shift it a bit */
        }
        """

        # QToolButton:pressed {
        #     background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1,
        #                                       stop: 0 #dadbde, stop: 1 #f6f7fa);
        # }

        self.setStyleSheet(stil)



# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__date__ = '02/Nov/2018'
__author__ = 'Erdinç Yılmaz'

import math
from PySide6.QtCore import Qt, QRectF, QSizeF, QPointF, QLineF, QBuffer, QIODevice
from PySide6.QtGui import QPainterPath, QPainterPathStroker, QPen, QColor, QTransform, QPolygonF, QTextOption, QBrush, \
    QPainter
from PySide6.QtSvg import QSvgGenerator
from PySide6.QtWidgets import QGraphicsItem, QStyle
from .. import shared
from ..nesneler.tempTextItem import TempTextItem


########################################################################
class LineItem(QGraphicsItem):
    Type = shared.LINE_ITEM_TYPE

    def __init__(self, pos, pen, yaziRengi=None, arkaPlanRengi=Qt.GlobalColor.transparent,
                 font=None, line=None,
                 parent=None):
        super(LineItem, self).__init__(parent)

        self._kim = shared.kim(kac_basamak=16)

        self.setAcceptHoverEvents(True)

        # cizim icin False baslamak dogru olabilirdi ama
        # kopyalama ve dosyadan yukleme gibi durumlarda bu false kalıyor
        # o yuzden True baslayip, cizim de ilk noktada(move_start_point) False ediyoruz
        self.isDrawingFinished = True

        self.okBoyutu = pen.widthF() * 3
        self.ok_polygon = QPolygonF()
        # self.ok_polygon << QPoint(-5, 0) << QPoint(5, 0) << QPoint(0, 5) << QPoint(-5, 0)

        self._line = QLineF()

        self.setPos(pos)

        self.secili_nesne_kalem_kalinligi = 0

        self.handleSize = 25
        self.resizeHandleSize = QSizeF(self.handleSize, self.handleSize)
        self.update_resize_handles()

        if line:
            self.setLine(line)

        self.initialFlags = self.flags()
        self.setFlags(QGraphicsItem.GraphicsItemFlag.ItemIsSelectable |
                      QGraphicsItem.GraphicsItemFlag.ItemIsMovable |
                      QGraphicsItem.GraphicsItemFlag.ItemIsFocusable)

        self._pen = pen
        self._font = font

        self.activeItemLineColor = shared.activeItemLineColor
        self.cizgiRengi = pen.color()
        self.setCizgiRengi(self.cizgiRengi)  # also sets self._pen
        self.yaziRengi = yaziRengi
        self.setYaziRengi(yaziRengi)
        self.arkaPlanRengi = arkaPlanRengi  # Ok-cizgi nesnesinde kullanilmiyor

        self.tempTextItem = None
        self.tempEskiText = ""
        self.textPadding = 2

        self.painterTextOption = QTextOption()
        self.painterTextOption.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.painterTextOption.setWrapMode(QTextOption.WrapMode.WrapAtWordBoundaryOrAnywhere)

        self.painterTextRect = QRectF()

        self.cosmeticSelect = False
        self.isActiveItem = False
        self._isPinned = False
        self.ustGrup = None

        self._text = ""
        self._command = {}

        # self.editMode = False

        self.baglanmis_nesneler = {}
        self.oklar_dxdy_nokta = {}

    # ---------------------------------------------------------------------
    def type(self):
        return LineItem.Type

    # ---------------------------------------------------------------------
    def get_properties_for_save_binary(self):
        properties = {"type": self.type(),
                      # "painterPath": self.path(),
                      "kim": self._kim,
                      "line": self._line,
                      "pos": self.pos(),
                      "rotation": self.rotation(),
                      "zValue": self.zValue(),
                      "pen": self._pen,
                      "font": self._font,
                      "yaziRengi": self.yaziRengi,
                      "arkaPlanRengi": self.arkaPlanRengi,
                      "yaziHiza": int(self.painterTextOption.alignment()),
                      "text": self.text(),
                      "isPinned": self.isPinned,
                      "command": self._command,
                      "baglanmis_nesneler": self.baglanmis_nesneler,
                      }
        return properties

    # ---------------------------------------------------------------------
    def ok_ekle(self, ok, scenepPos, okunHangiNoktasi):

        # self.oklar_dxdy_nokta.append((ok, self.mapFromScene(scenePos)))
        sceneCenter = self.sceneCenter()
        dx = sceneCenter.x() - scenepPos.x()
        dy = sceneCenter.y() - scenepPos.y()

        # self.oklar_dxdy_nokta[ok._kim] = (dx, dy, nokta)
        self.oklar_dxdy_nokta[ok] = (dx, dy, okunHangiNoktasi)

        ok.baglanmis_nesneler[self._kim] = okunHangiNoktasi

    # ---------------------------------------------------------------------
    def ok_sil(self, ok):
        del self.oklar_dxdy_nokta[ok]
        del ok.baglanmis_nesneler[self._kim]

    # ---------------------------------------------------------------------
    def ok_guncelle(self):
        if self.oklar_dxdy_nokta:
            sceneCenter = self.sceneCenter()
            scx = sceneCenter.x()
            scy = sceneCenter.y()
            for ok, dxdy_nokta in self.oklar_dxdy_nokta.items():
                if dxdy_nokta[2] == 1:
                    ok.temp_prepend(QPointF(scx - dxdy_nokta[0], scy - dxdy_nokta[1]))
                elif dxdy_nokta[2] == 2:
                    ok.temp_append(QPointF(scx - dxdy_nokta[0], scy - dxdy_nokta[1]))
                if ok._text:
                    ok.update_painter_text_rect()

    # ---------------------------------------------------------------------
    def varsaEnUsttekiGrubuGetir(self):
        parentItem = self.parentItem()
        while parentItem:
            if parentItem.type() == shared.GROUP_ITEM_TYPE:
                return parentItem
            parentItem = parentItem.parentItem()
        return None

    # ---------------------------------------------------------------------
    def move_start_point(self):
        self.isDrawingFinished = False
        # point = self.mapFromScene(point)
        point = QPointF(0, 0)
        self._line.setP1(point)

    # ---------------------------------------------------------------------
    def temp_prepend(self, point):
        point = self.mapFromScene(point)
        line = QLineF(self._line)
        line.setP1(point)
        self.setLine(line)
        self.update_resize_handles()

    # ---------------------------------------------------------------------
    def temp_append(self, point):
        point = self.mapFromScene(point)
        line = QLineF(self._line)
        line.setP2(point)
        self.setLine(line)
        self.update_resize_handles()
        sahne_acisi = f"\u2220  {((self._line.angle() - self.rotation()) % 360):.1f}\N{DEGREE SIGN}"
        self.scene().parent().log(txt=sahne_acisi, toStatusBarOnly=True)

    # ---------------------------------------------------------------------
    def update_resize_handles(self):
        self.prepareGeometryChange()

        if self.scene():
            resizeHandleSize = self.resizeHandleSize / self.scene().views()[0].transform().m11()
        else:
            resizeHandleSize = self.resizeHandleSize

        self.p1Handle = QRectF(self._line.p1(), resizeHandleSize)
        self.p2Handle = QRectF(self._line.p2(), resizeHandleSize)

        self.p1Handle.moveCenter(self._line.p1())
        self.p2Handle.moveCenter(self._line.p2())

    # ---------------------------------------------------------------------
    def hide_resize_handles(self):
        self.isPinned = True

    # ---------------------------------------------------------------------
    def show_resize_handles(self):
        self.isPinned = False

    # ---------------------------------------------------------------------
    def setArkaPlanRengi(self, col):
        # arkaPlanRengi Ok-Cizgi Nesnesinde kullanilmiyor
        # fasulyeden method
        self.arkaPlanRengi = col
        # self.setBrush(QBrush(col))
        # self.update()

    # ---------------------------------------------------------------------
    def brush(self):
        # bu nesnede brush kullanmiyoruz ama diger nesnelerin ilgili methodlariyla uyumlu olsun diye
        # ekliyoruz, setArkaplanRengi de bu minvalde
        # return Qt.NoBrush
        return QBrush()

    # ---------------------------------------------------------------------
    def is_temp_text_item_empty(self):
        tmpText = self.tempTextItem.toPlainText()
        self.scene().undoRedo.undoableItemSetText(self.scene().undoStack, "change text", self,
                                                  self.tempEskiText, tmpText)
        # if not tmpText:
        self.scene().removeItem(self.tempTextItem)
        self.tempTextItem.deleteLater()
        self.tempTextItem = None

    # ---------------------------------------------------------------------
    def mouseDoubleClickEvent(self, event):
        # eger cizim yapiyorken , yani daha sahneye resmi olarak eklemedik
        # o zaman cizimibitirmeden cift tiklarsak cokgen nesnesinin yazi yazma kutucugu aciliyor
        # ama sonrasi problem, o yuzden asagidaki yazi yazma ozelligini eger parent yoksa demekki sahneye eklenmemis
        # diye dusunerek aktif etmiyoruz.
        # AYRICA SANHE DOUBLE CLICK inde, normal text nesnesi eklemse ile ilgili durum kontrol ediliyor.
        if self.isDrawingFinished:
            if not self.tempTextItem:
                self.tempTextItem = TempTextItem(self.sceneWidth() - self.textPadding,
                                                 self._font,
                                                 self.yaziRengi,
                                                 parent=self)
                self.tempTextItem.setParentItem(self)

            self.tempTextItem.setPlainText(self.text())
            self.tempEskiText = self.text()
            self.setText("")
            self.tempTextItem.setRotation(360 - self.rotation())
            # self.tempTextItem.rotateWithOffset(360 - self.rotation())
            self.tempTextItem.setCenter(self.boundingRect().center())
            self.tempTextItem.setFocus()

            # !! burda lambda kullanmazsak, self.is_temp_text_item_empty ,self daha olsumadigi icin,
            # selfi null olarak goruyor ve baglanti kuramiyor.
            self.tempTextItem.textItemFocusedOut.connect(lambda: self.is_temp_text_item_empty())
            self.update_painter_text_rect()

        super(LineItem, self).mouseDoubleClickEvent(event)

    # ---------------------------------------------------------------------
    def mousePressEvent(self, event):

        self._resizing = False  # we could use "self._resizingFrom = 0" instead, but self._resizing is more explicit.
        self._fixedResizePoint = None
        self._resizingFrom = None
        self._eskiLineBeforeResize = None

        self._eskiPosBeforeResize = self.pos()

        if self.p1Handle.contains(event.pos()):
            self._resizing = True
            self._fixedResizePoint = self._line.p2()
            self._resizingFrom = 1
        elif self.p2Handle.contains(event.pos()):
            self._resizing = True
            self._fixedResizePoint = self._line.p1()
            self._resizingFrom = 2
        self._eskiLineBeforeResize = self._line

        super(LineItem, self).mousePressEvent(event)

        # TODO: bu kozmetik amacli, sadece release de olunca gecikmeli seciyormus hissi oluyor
        if self.isSelected() and self.childItems():
            self.scene().select_all_children_recursively(self, cosmeticSelect=True)

    # ---------------------------------------------------------------------
    def mouseReleaseEvent(self, event):
        if self._resizing:
            # sahneye ilk cizim kontrolu,(undoya eklemesin diye)
            if not self._eskiLineBeforeResize.length() == 0:
                line = self._line
                self.scene().undoRedo.undoableResizeLineItem(self.scene().undoStack,
                                                             "resize",
                                                             self,
                                                             # yeniRect=self.rect(),
                                                             yeniLine=line,
                                                             eskiLine=self._eskiLineBeforeResize,
                                                             eskiPos=self._eskiPosBeforeResize,
                                                             degisenNokta=self._resizingFrom)
            # # TODO: bu satir eger cizim disari tasarsa, scene recti ona uygun ayarlamak icin
            # fakat eger cizim disari tasarsa evet uyarliyor ama bazen de çizilen itemin geometrisini degistiriyor.
            # o yuzden simdilik iptal.
            # self.scene().unite_with_scene_rect(self.sceneBoundingRect())
            self._resizing = False
            self.scene().unite_with_scene_rect(self.sceneBoundingRect())
            self.update_painter_text_rect()

        super(LineItem, self).mouseReleaseEvent(event)

        if self.childItems():
            if self.isSelected():
                self.scene().select_all_children_recursively(self, cosmeticSelect=False, topmostParent=True)
            else:
                self.scene().select_all_children_recursively(self, cosmeticSelect=False)

    # ---------------------------------------------------------------------
    def mouseMoveEvent(self, event):

        if self._resizing:

            if self._resizingFrom == 1:
                p1 = event.pos()
                p2 = self._fixedResizePoint

            elif self._resizingFrom == 2:
                p1 = self._fixedResizePoint
                p2 = event.pos()

            line = QLineF(p1, p2)

            # shift ile kullaniyoruz
            eski_aci = self._eskiLineBeforeResize.angle()

            # Alt Key - to resize around center.
            if event.modifiers() == Qt.KeyboardModifier.AltModifier:
                # burda da orta nokta sabit
                # c = self._line.center() # ile
                # ve de eski aci ile, belki bir cizgi hareket eden noktadan merkeze
                # sonra da length *2, tabi hangi nokta tutuldu ise p2, p2 ona gore ayarlanmali ok icin
                pass

            # ---------------------------------------------------------------------
            #  Ctrl Key - 45 derece ve katlarina kilitlemek icin
            if event.modifiers() == Qt.KeyboardModifier.ControlModifier:
                if self._resizingFrom == 1:
                    line = QLineF(line.p2(), line.p1())

                if line.dx():  # zero division error
                    yeni_aci = 0
                    aci = math.atan2(-line.dy(), line.dx())
                    # print(aci)
                    aci = aci * 180 / math.pi
                    # print(aci)

                    if -180 <= aci < -165:
                        yeni_aci = 180
                    elif -165 <= aci < -120:
                        yeni_aci = 225
                    elif -120 <= aci < -75:
                        yeni_aci = 270
                    elif -75 <= aci < -30:
                        yeni_aci = 315
                    if -30 <= aci < 30:
                        yeni_aci = 0
                    elif 30 <= aci < 75:
                        yeni_aci = 45
                    elif 75 <= aci < 120:
                        yeni_aci = 90
                    elif 120 <= aci < 165:
                        yeni_aci = 135
                    elif aci >= 165:
                        yeni_aci = 180

                    line.setAngle(yeni_aci)

                    if self._resizingFrom == 1:
                        line = QLineF(line.p2(), line.p1())

            # ---------------------------------------------------------------------
            # Shift Key - cizgi uzunlugu değiştiriken açıyı değiştirmek için
            if event.modifiers() == Qt.KeyboardModifier.ShiftModifier:
                if self._resizingFrom == 1:
                    line = QLineF(line.p2(), line.p1())
                    line.setAngle(180 + eski_aci)
                    line = QLineF(line.p2(), line.p1())
                elif self._resizingFrom == 2:
                    line.setAngle(eski_aci)

            self.setLine(line)  # mouse release eventten gonderiyoruz undoya
            self.update_resize_handles()
            self.update_painter_text_rect()
            self.scene().parent().change_transform_box_values(self)

            # TODO: self.rotation() degeri, self._line.angle() ile degissin, yani setPos ile
            # pozisyonu da ayarlamak lazım.

            sahne_acisi = f"\u2220  {((self._line.angle() - self.rotation()) % 360):.1f}\N{DEGREE SIGN}"
            self.scene().parent().log(txt=sahne_acisi, toStatusBarOnly=True)
            # self.setPos(x, y)
            # return QGraphicsItem.mouseMoveEvent(self, event)

        # event.accept()
        else:
            super(LineItem, self).mouseMoveEvent(event)

        # super(BaseItem, self).mouseMoveEvent(event)

    # ---------------------------------------------------------------------
    def hoverEnterEvent(self, event):
        # event propagationdan dolayi bos da olsa burda implement etmek lazim
        # mousePress,release,move,doubleclick de bu mantıkta...
        # baska yerden baslayip mousepress ile , mousemove baska, mouseReleas baska widgetta gibi
        # icinden cikilmaz durumlara sebep olabiliyor.
        # ayrica override edince accept() de edilmis oluyor mouseeventler
        super(LineItem, self).hoverEnterEvent(event)

    # ---------------------------------------------------------------------
    def hoverMoveEvent(self, event):
        # self.sagUstKare.hide()

        # cursor = self.scene().parent().cursor()

        # if self.isSelected():
        if not self.ustGrup:
            if self.scene().aktifArac == self.scene().SecimAraci:
                if self.p1Handle.contains(event.pos()) or self.p2Handle.contains(event.pos()):
                    self.scene().parent().setCursor(Qt.CursorShape.SizeAllCursor, gecici_mi=True)
                else:
                    self.scene().parent().setCursor(self.scene().parent().imlec_arac, gecici_mi=True)

        super(LineItem, self).hoverMoveEvent(event)

    # ---------------------------------------------------------------------
    def hoverLeaveEvent(self, event):
        if not self.ustGrup:
            self.scene().parent().setCursor(self.scene().parent().imlec_arac, gecici_mi=True)

        super(LineItem, self).hoverLeaveEvent(event)

    # ---------------------------------------------------------------------
    @property
    def isPinned(self):
        return self._isPinned

    # ---------------------------------------------------------------------
    @isPinned.setter
    def isPinned(self, value):
        if value:
            self.setFlags(QGraphicsItem.GraphicsItemFlag.ItemIsSelectable
                          # | QGraphicsItem.GraphicsItemFlag.ItemIsMovable
                          #  | QGraphicsItem.GraphicsItemFlag.ItemIsFocusable
                          )

        else:
            self.setFlags(QGraphicsItem.GraphicsItemFlag.ItemIsSelectable
                          | QGraphicsItem.GraphicsItemFlag.ItemIsMovable
                          | QGraphicsItem.GraphicsItemFlag.ItemIsFocusable)
        self._isPinned = value

    # ---------------------------------------------------------------------
    def itemChange(self, change, value):
        # if change == self.ItemPositionChange:
        # if change == self.ItemSelectedChange:
        # if change == self.ItemSelectedHasChanged:
        if change == QGraphicsItem.GraphicsItemChange.ItemSelectedChange:
            if value:
                self.scene().parent().line_item_selected(self)
            else:
                self.scene().parent().item_deselected(self)

        # return QGraphicsRectItem.itemChange(self, change, value)
        return value
        # super(Rect, self).itemChange(change, value)

    # ---------------------------------------------------------------------
    def contextMenuEvent(self, event):
        # TODO: bu alttaki iki satir aktif idi, ve de ctrl harici sag tiklayinca
        # (base - text - path -group nesnelerinin contextMenuEventinde var)
        # mesela birden fazla nesne secili ve de gruplayacagız diyelim sag menu ile
        # ctrl basılı degil ise tikladigimiz secili kaliyor digerleri siliniyordu
        # uygunsuz bir kullanıcı deneyimi, niye yaptık acaba boyleyi hatırlayana kadar kalsın burda :)
        # if not event.modifiers() == Qt.KeyboardModifier.ControlModifier:
        #     self.scene().clearSelection()
        self.setSelected(True)

        # self.scene().parent().item_context_menu(self, event.screenPos())
        self.scene().parent().on_nesne_sag_menu_about_to_show(self)
        self.scene().parent().nesneSagMenu.popup(event.screenPos())

        event.accept()

    # ---------------------------------------------------------------------
    def sceneCenter(self):
        if self.parentItem():
            return self.mapToParent(self._line.center())
        return self.mapToScene(self._line.center())

    # ---------------------------------------------------------------------
    def sceneWidth(self):
        return self.sceneRight() - self.sceneLeft()

    # ---------------------------------------------------------------------
    def sceneHeight(self):
        return self.sceneBottom() - self.sceneTop()

    # ---------------------------------------------------------------------
    def sceneRight(self):

        # p1 = self.mapToScene(self._line.p1())
        # p2 = self.mapToScene(self._line.p2())
        # return max(p1.x(), p2.x())

        # kalem ve ok ucu kalingligi dahil edildi
        rect = self.mapRectToScene(self.shape().controlPointRect())
        return rect.right()

    # ---------------------------------------------------------------------
    def sceneLeft(self):

        # p1 = self.mapToScene(self._line.p1())
        # p2 = self.mapToScene(self._line.p2())
        # return min(p1.x(), p2.x())

        # kalem ve ok ucu kalingligi dahil edildi
        rect = self.mapRectToScene(self.shape().controlPointRect())
        return rect.left()

    # ---------------------------------------------------------------------
    def sceneTop(self):
        # p1 = self.mapToScene(self._line.p1())
        # p2 = self.mapToScene(self._line.p2())
        # return min(p1.y(), p2.y())

        # kalem ve ok ucu kalingligi dahil edildi
        rect = self.mapRectToScene(self.shape().controlPointRect())
        return rect.top()

    # ---------------------------------------------------------------------
    def sceneBottom(self):
        # p1 = self.mapToScene(self._line.p1())
        # p2 = self.mapToScene(self._line.p2())
        # return max(p1.y(), p2.y())

        # kalem ve ok ucu kalingligi dahil edildi
        rect = self.mapRectToScene(self.shape().controlPointRect())
        return rect.bottom()

    # ---------------------------------------------------------------------
    def setSceneLeft(self, left):
        self.setX(left + self.scenePos().x() - self.sceneLeft())

    # ---------------------------------------------------------------------
    def setSceneRight(self, right):
        self.setX(right + self.scenePos().x() - self.sceneRight())

    # ---------------------------------------------------------------------
    def setSceneTop(self, top):
        self.setY(top + self.scenePos().y() - self.sceneTop())

    # ---------------------------------------------------------------------
    def setSceneBottom(self, bottom):
        self.setY(bottom + self.scenePos().y() - self.sceneBottom())

    # ---------------------------------------------------------------------
    def setHorizontalCenter(self, hcenter):
        self.setY(hcenter + self.scenePos().y() - self.sceneCenter().y())

    # ---------------------------------------------------------------------
    def setVerticalCenter(self, vcenter):
        self.setX(vcenter + self.scenePos().x() - self.sceneCenter().x())

    # ---------------------------------------------------------------------
    def length(self):
        return self._line.length()

    # ---------------------------------------------------------------------
    def line(self):
        return self._line

    # ---------------------------------------------------------------------
    def setLine(self, line):
        if self._line == line:
            return
        self._line = line
        self._boundingRect = QRectF()
        self.ok_ucu_guncelle()
        self.prepareGeometryChange()
        self.update()

    # ---------------------------------------------------------------------
    def ok_ucu_guncelle(self):
        # line = self._line
        line = QLineF(self._line)
        try:
            angle = math.acos(line.dx() / line.length())
        except ZeroDivisionError:
            return

        if line.dy() >= 0:
            angle = (math.pi * 2.0) - angle

        # line.setLength(line.length() + self.okBoyutu * .87)
        line.setLength(line.length() + self.okBoyutu * 0.45)

        okP1 = line.p2() - QPointF(math.sin(angle + math.pi / 3.0) * self.okBoyutu,
                                   math.cos(angle + math.pi / 3.0) * self.okBoyutu)
        okP2 = line.p2() - QPointF(math.sin(angle + math.pi - math.pi / 3.0) * self.okBoyutu,
                                   math.cos(angle + math.pi - math.pi / 3.0) * self.okBoyutu)

        self.ok_polygon.clear()
        for point in [line.p2(), okP1, okP2]:
            self.ok_polygon.append(point)
        self.update_resize_handles()

    # ---------------------------------------------------------------------
    def setCizgiRengi(self, col):
        _selectionLineBgColor = QColor.fromHsv(col.hue(),
                                               0,
                                               20 if col.value() > 127 else 250)

        _activeItemLineColor = QColor(self.activeItemLineColor)
        if col.hue() > 300 or col.hue() < 30:
            _activeItemLineColor.setHsv((col.hue() + 150) % 360,
                                        self.activeItemLineColor.saturation(),
                                        self.activeItemLineColor.value())
        self._pen.setColor(col)
        self.cizgiRengi = col

        self.selectionPenBottom = QPen(_selectionLineBgColor,
                                       self.secili_nesne_kalem_kalinligi,
                                       Qt.PenStyle.DashLine,
                                       Qt.PenCapStyle.RoundCap, Qt.PenJoinStyle.RoundJoin)
        self.selectionPenBottomIfAlsoActiveItem = QPen(_activeItemLineColor,
                                                       self.secili_nesne_kalem_kalinligi,
                                                       Qt.PenStyle.DashLine,
                                                       Qt.PenCapStyle.RoundCap, Qt.PenJoinStyle.RoundJoin)

        self.update()

    # ---------------------------------------------------------------------
    def setCizgiKalinligi(self, width):
        self._pen.setWidthF(width)
        self.okBoyutu = width * 3
        self.ok_ucu_guncelle()

        self.update()

    # ---------------------------------------------------------------------
    def setYaziRengi(self, col):
        # drawing text after drawing rect does not apply alpha
        # we need to reconstruct the color with same values.
        self.yaziRengi = col
        self.update()

    # ---------------------------------------------------------------------
    def setText(self, text):
        self._text = text
        # self.update()
        self.update_painter_text_rect()

    # ---------------------------------------------------------------------
    def text(self):
        return self._text

    # ---------------------------------------------------------------------
    def setCommand(self, title, command):
        self._command["title"] = title
        self._command["command"] = command

    # ---------------------------------------------------------------------
    def command(self):
        return self._command

    # ---------------------------------------------------------------------
    def setFont(self, font):
        font.setBold(self._font.bold())
        font.setItalic(self._font.italic())
        font.setUnderline(self._font.underline())
        font.setStrikeOut(self._font.strikeOut())
        font.setOverline(self._font.overline())
        font.setPointSizeF(self._font.pointSizeF())
        self._font = font
        self.update()

    # ---------------------------------------------------------------------
    def font(self):
        return self._font

    # ---------------------------------------------------------------------
    def setFontPointSizeF(self, fontPointSizeF):
        self._font.setPointSizeF(fontPointSizeF)
        self.update()

    # ---------------------------------------------------------------------
    def fontPointSizeF(self):
        return self._font.pointSizeF()

    # ---------------------------------------------------------------------
    def ver_yazi_hizasi(self):
        return self.painterTextOption.alignment()

    # ---------------------------------------------------------------------
    def kur_yazi_hizasi(self, hizalama):
        self.painterTextOption.setAlignment(hizalama | Qt.AlignmentFlag.AlignVCenter)
        # self.painterTextOption.setAlignment(hizalama)
        self.update()

    # ---------------------------------------------------------------------
    def ver_karakter_bicimi(self):
        # return self._font.bold | self._font.italic | self._font.under | self._font.strikeout
        return {"b": self._font.bold(),
                "i": self._font.italic(),
                "u": self._font.underline(),
                "s": self._font.strikeOut(),
                "o": self._font.overline(),
                }

    # ---------------------------------------------------------------------
    def kur_karakter_bicimi(self, sozluk):
        self._font.setBold(sozluk["b"])
        self._font.setItalic(sozluk["i"])
        self._font.setUnderline(sozluk["u"])
        self._font.setStrikeOut(sozluk["s"])
        self._font.setOverline(sozluk["o"])
        self.update()

    # ---------------------------------------------------------------------
    def update_painter_text_rect(self):
        r = QRectF()
        r.setWidth(self._line.length())
        # r.setWidth(max(self._line.length(), self.fontPointSizeF()))
        # r.setHeight(self.fontPointSizeF()- self.textPadding)
        # r.setHeight(self.fontPointSizeF() + 30)
        # fontPixelSize = self.fontPointSizeF() / 72 * self.scene().parent().dpi
        r.setHeight(self.fontPointSizeF())
        # r.moveCenter(self.boundingRect().center())
        yeniOrtaNokta = self._line.center()
        yeniOrtaNokta.setY(yeniOrtaNokta.y() - self._pen.widthF())
        # yeniOrtaNokta.setY(yeniOrtaNokta.y() - r.height())
        r.moveCenter(yeniOrtaNokta)
        self.painterTextRect = r

    # ---------------------------------------------------------------------
    def setPen(self, pen):

        if self._pen == pen:
            return
        self.prepareGeometryChange()
        self._pen = pen
        self._boundingRect = QRectF()
        self.update(self.boundingRect())

    # ---------------------------------------------------------------------
    def pen(self):
        return self._pen

    # ---------------------------------------------------------------------
    def flipHorizontal(self, mposx):
        lc = self.mapToScene(self._line.center())
        diff = mposx - lc.x()
        ipos = self.scenePos()
        ipos.setX(ipos.x() + 2 * diff)

        p1 = self._line.p1()
        p2 = self._line.p2()
        eskiP1x = p1.x()
        p1.setX(p2.x())
        p2.setX(eskiP1x)
        self.setLine(QLineF(p1, p2))

        if self.parentItem():
            # eger self parented ise, parented degilmis gibi dusunup
            # sahne pozisyonlari ile mposxe gore yeni mirror pos unu bulup
            # buldugumuzdan sahne pos unu cıkarıp set ediyoruz.
            # bu methodu grup cagirdiginda, donusunde grup icinde bounding rect
            # tekrar hesaplandigi icin, Hamd Olsun islem tamam oluyor.
            # (ayrica grup rotation sifir degilse gecici olarak sifir yapiliyor.
            # burdaki islemler sifir rotationda oluyor. sonra bitince 360- origRot
            # ile grup flipped rotationa donduruluyor)
            # dolayısı ile burda grubun pos u ile ilgili baska bir isleme gerek kalmiyor.
            # ya da grup icersinde. yani grubu icerige giydiriyoruz. islem bitince.

            # ipos = (ipos - self.parentItem().scenePos())
            # pass
            ipos = (self.parentItem().mapFromScene(ipos))
            # ipos = self.parentItem().transform().map(ipos)

        self.setPos(ipos)
        if self.rotation():
            self.rotateWithOffset(360 - self.rotation())

        eskiRot = self.rotation()
        if eskiRot:
            self.rotateWithOffset(0)
        # eger item parent ise, icindekileri de flip etmek icin,
        # oncesinde yine item rotationu sifirladik, icindekileri flip edince, yine eski rotationa donduruyoruz.
        # bu bi anlamda recursive bir method, neden yukardaki if self.parentItem() i kullanmadik
        # az sonra cagirdigimizda yine bu fonksiyonu, yani child itemin rotation hesaplamalari icin,
        # cunku burda yapmak daha basit ve daha hizli, bu halde iken hesaplamalari c++ yapiyor, eger az once
        # bahsedilen yerde yapilirsa her item icin python yapacak.
        for c in self.childItems():
            c.flipHorizontal(self.sceneCenter().x())
        if eskiRot:
            self.rotateWithOffset(eskiRot)

    # ---------------------------------------------------------------------
    def flipVertical(self, mposy):
        lc = self.mapToScene(self._line.center())
        diff = mposy - lc.y()
        ipos = self.scenePos()
        ipos.setY(ipos.y() + 2 * diff)

        p1 = self._line.p1()
        p2 = self._line.p2()
        eskiP1y = p1.y()
        p1.setY(p2.y())
        p2.setY(eskiP1y)
        self.setLine(QLineF(p1, p2))

        if self.parentItem():
            # ipos = (ipos - self.parentItem().scenePos())
            ipos = (self.parentItem().mapFromScene(ipos))

        self.setPos(ipos)
        if self.rotation():
            self.rotateWithOffset(360 - self.rotation())

        eskiRot = self.rotation()
        if eskiRot:
            self.rotateWithOffset(0)
        for c in self.childItems():
            c.flipVertical(self.sceneCenter().y())
        if eskiRot:
            self.rotateWithOffset(eskiRot)

    # ---------------------------------------------------------------------
    def qt_graphicsItem_shapeFromPath(self, path, pen):

        # We unfortunately need this hack as QPainterPathStroker will set a width of 1.0
        # if we pass a value of 0.0 to QPainterPathStroker.setWidth()
        penWidthZero = 0.00000001
        if path == QPainterPath():
            return path
        ps = QPainterPathStroker()
        ps.setCapStyle(pen.capStyle())

        if pen.widthF() <= 0:
            ps.setWidth(penWidthZero)
        else:
            ps.setWidth(pen.widthF())
        ps.setJoinStyle(pen.joinStyle())
        ps.setMiterLimit(pen.miterLimit())
        p = ps.createStroke(path)  # returns QPainterPath
        p.addPath(path)
        return p

    # ---------------------------------------------------------------------
    def boundingRect(self):

        """
        QRectF QGraphicsLineItem::boundingRect() const
        {
            Q_D(const QGraphicsLineItem);
            if (d->pen.widthF() == 0.0) {
                const qreal x1 = d->line.p1().x();
                const qreal x2 = d->line.p2().x();
                const qreal y1 = d->line.p1().y();
                const qreal y2 = d->line.p2().y();
                qreal lx = qMin(x1, x2);
                qreal rx = qMax(x1, x2);
                qreal ty = qMin(y1, y2);
                qreal by = qMax(y1, y2);
                return QRectF(lx, ty, rx - lx, by - ty);
            }
            return shape().controlPointRect();
        }

        """
        rect = self.shape().controlPointRect()
        return rect

    # ---------------------------------------------------------------------
    def shape(self):

        path = QPainterPath()
        if self._line == QLineF():
            return path
        path.moveTo(self._line.p1())
        path.lineTo(self._line.p2())
        path.addPolygon(self.ok_polygon)
        path.addRect(self.p1Handle)
        path.addRect(self.p2Handle)

        if self._text:
            # path addText kullanmadik, ok uzunlugu kısalınca yazının hepsini gostermese de kareye ekliyor.
            # painterTextRecti okla beraber dondurup donmus dortgenin boundinRectini alip ekliyoruz
            # c = self._line.center()
            # t = QTransform().translate(c.x(), c.y()).rotate(- self._line.angle()).translate(-c.x(), -c.y())
            # self.rpoly = t.mapToPolygon(self.painterTextRect.toRect())
            # path.addRect(self.rpoly.boundingRect())

            path.addRect(self.painterTextRect)

        # kalem kalinligini da ekliyoruz
        path = self.qt_graphicsItem_shapeFromPath(path, self.pen())
        return path

    # ---------------------------------------------------------------------
    def wheelEvent(self, event):
        # factor = 1.41 ** (event.delta() / 240.0)

        if self.ustGrup:
            return QGraphicsItem.wheelEvent(self, event)

        toplam = event.modifiers().value

        # ctrl = int(Qt.ControlModifier)
        shift = Qt.KeyboardModifier.ShiftModifier.value
        alt = Qt.KeyboardModifier.AltModifier.value

        ctrlAlt = Qt.KeyboardModifier.ControlModifier.value + Qt.KeyboardModifier.AltModifier.value
        ctrlShift = Qt.KeyboardModifier.ControlModifier.value + Qt.KeyboardModifier.ShiftModifier.value
        altShift = Qt.KeyboardModifier.AltModifier.value + Qt.KeyboardModifier.ShiftModifier.value
        ctrlAltShift = Qt.KeyboardModifier.ControlModifier.value + Qt.KeyboardModifier.AltModifier.value + Qt.KeyboardModifier.ShiftModifier.value

        # if event.modifiers() & == Qt.KeyboardModifier.ControlModifier:
        if toplam == ctrlShift:
            self.scaleItemByScalingLine(event.delta())

        # elif event.modifiers() == Qt.KeyboardModifier.ShiftModifier:
        elif toplam == shift:
            self.changeFontSizeF(event.delta())

        # elif event.modifiers() == Qt.KeyboardModifier.AltModifier:
        elif toplam == alt:
            # self.changeBackgroundColorAlpha(event.delta())
            self.changeLineColorAlpha(event.delta())

        elif toplam == ctrlAlt:
            self.changeTextColorAlpha(event.delta())

        elif toplam == ctrlAltShift:
            self.rotateItem(event.delta())

        elif toplam == altShift:
            if self.isDrawingFinished:
                self._cizgi_kalinligi_degistir_undo_ile(event.delta())
            # else:
            # self.scene().tekerlek_ile_firca_boyu_degistir(event.delta())

        if not self.isDrawingFinished:
            self.scene().tekerlek_ile_firca_boyu_degistir(event.delta())

        # elif toplam == ctrlAltShift:
        #     self.scaleItemByScalingLine(event.delta())

        else:
            super(LineItem, self).wheelEvent(event)

    # ---------------------------------------------------------------------
    def changeFontSizeF(self, delta):

        if self.tempTextItem:
            return

        # font = self.font()
        size = self.fontPointSizeF()

        if delta > 0:
            # font.setPointSizeF(size + 1)
            size += 1

        else:
            if size > 10:
                # font.setPointSizeF(size - 1)
                size -= 1
            else:
                # undolari biriktermesin diye donuyoruz,
                # yoksa zaten ayni size de yeni bir undolu size komutu veriyor.
                return

        # self.setFont(font)
        if self.childItems():
            self.scene().undoStack.beginMacro("change text size")
            self.scene().undoRedo.undoableSetFontSizeF(self.scene().undoStack, "change text size", self, size)
            for c in self.childItems():
                c.changeFontSizeF(delta)
            self.scene().undoStack.endMacro()
        else:
            self.scene().undoRedo.undoableSetFontSizeF(self.scene().undoStack, "change text size", self, size)

    # ---------------------------------------------------------------------
    def scaleItemByScalingLine(self, delta):

        if self.tempTextItem:
            return

        scaleFactor = 1.1
        if delta < 0:
            scaleFactor = 1 / scaleFactor

        scaleMatrix = QTransform()
        scaleMatrix.scale(scaleFactor, scaleFactor)
        scaledLine = self._line * scaleMatrix
        # if scaledLine.length() < 5:
        #     return
        self.scene().undoRedo.undoableScaleLineItemByScalingLine(self.scene().undoStack,
                                                                 "scale",
                                                                 self, scaledLine,
                                                                 scaleFactor)

        # self.scene().unite_with_scene_rect(self.sceneBoundingRect())
        # self.setTransformOriginPoint(self.boundingRect().center())

    # ---------------------------------------------------------------------
    def repositionChildItems(self, diff):
        # TODO: item rotation 0 olmadigi durumlarda calismiyor , kaymalar oluyor.
        for c in self.childItems():
            c.moveBy(-diff.x(), -diff.y())

    # ---------------------------------------------------------------------
    def rotateWithOffset(self, angle):
        cEski = self.sceneCenter()
        self.setRotation(angle)
        cYeni = self.sceneCenter()
        diff = cEski - cYeni
        self.moveBy(diff.x(), diff.y())
        self.update_painter_text_rect()

    # ---------------------------------------------------------------------
    def rotateItem(self, delta):
        if not self.isDrawingFinished:
            return

        # self.setTransformOriginPoint(self.boundingRect().center())
        if delta > 0:
            # self.setRotation(self.rotation() + 5)
            # bu p1 den rotate ediyor
            # self.scene().undoableRotate("Rotate", self, self.rotation() + 5)
            self.scene().undoRedo.undoableRotateWithOffset(self.scene().undoStack, "rotate", self, self.rotation() + 5)
        else:
            # self.setRotation(self.rotation() - 5)
            # bu p1 den rotate ediyor
            # self.scene().undoableRotate("Rotate", self, self.rotation() - 5)
            self.scene().undoRedo.undoableRotateWithOffset(self.scene().undoStack, "rotate", self, self.rotation() - 5)
        # self.update()
        # self.update_painter_text_rect()

    # # ---------------------------------------------------------------------
    # def changeBackgroundColorAlpha(self, delta):
    #
    #     col = shared.calculate_alpha(delta, QColor(self.arkaPlanRengi))
    #
    #     # self.setBrush(self.arkaPlanRengi)
    #     # self.update()
    #     # self.scene().undoableSetItemBackgroundColorAlpha("Change item's background alpha", self, col)
    #
    #     if self.childItems():
    #         self.scene().undoStack.beginMacro("change items' background alpha")
    #         self.scene().undoRedo.undoableSetItemBackgroundColorAlpha(self.scene().undoStack,
    #                                                                   "_change item's background alpha",
    #                                                                   self, col)
    #         for c in self.childItems():
    #             c.changeBackgroundColorAlpha(delta)
    #         self.scene().undoStack.endMacro()
    #     else:
    #         self.scene().undoRedo.undoableSetItemBackgroundColorAlpha(self.scene().undoStack,
    #                                                                   "change item's background alpha",
    #                                                                   self, col)
    # ---------------------------------------------------------------------
    def _cizgi_kalinligi_degistir_undo_ile(self, delta):

        if delta > 0:
            cap = self.scene().parent().fircaBuyult()
        else:
            cap = self.scene().parent().fircaKucult()

        self.scene().undoRedo.undoableSetLineWidthF(self.scene().undoStack, "change item's line width", self,
                                                    cap)

    # ---------------------------------------------------------------------
    def changeLineColorAlpha(self, delta):

        if self.tempTextItem:
            return

        col = shared.calculate_alpha(delta, self._pen.color())

        # self.scene().undoableSetLineOrTextColorAlpha("Change item's line alpha", self, col)
        if self.childItems():
            self.scene().undoStack.beginMacro("change items' line alpha")
            self.scene().undoRedo.undoableSetLineColorAlpha(self.scene().undoStack, "_change item's line alpha", self,
                                                            col)
            for c in self.childItems():
                c.changeLineColorAlpha(delta)
            self.scene().undoStack.endMacro()
        else:
            self.scene().undoRedo.undoableSetLineColorAlpha(self.scene().undoStack, "change item's line alpha", self,
                                                            col)

    # ---------------------------------------------------------------------
    def changeTextColorAlpha(self, delta):

        if self.tempTextItem:
            return

        col = shared.calculate_alpha(delta, self.yaziRengi)

        if self.childItems():
            self.scene().undoStack.beginMacro("change items' text alpha")
            self.scene().undoRedo.undoableSetTextColorAlpha(self.scene().undoStack, "_change item's text alpha", self,
                                                            col)
            for c in self.childItems():
                c.changeTextColorAlpha(delta)
            self.scene().undoStack.endMacro()
        else:
            self.scene().undoRedo.undoableSetTextColorAlpha(self.scene().undoStack, "change item's text alpha", self,
                                                            col)

    # ---------------------------------------------------------------------
    def changeImageItemTextBackgroundColorAlpha(self, delta):
        # grup nesnesinden cagriliyor alt+shift
        if self.childItems():
            startedMacro = False
            for c in self.childItems():
                if c.type() == shared.IMAGE_ITEM_TYPE:
                    if not startedMacro:
                        self.scene().undoStack.beginMacro("change image items' text background alpha")
                        startedMacro = True
                    c.changeImageItemTextBackgroundColorAlpha(delta)
            if startedMacro:
                self.scene().undoStack.endMacro()

    # ---------------------------------------------------------------------
    def paint(self, painter, option, widget=None):
        painter.setPen(self._pen)
        painter.drawLine(self._line)
        painter.setBrush(self._pen.color())
        # painter.drawEllipse(self._line.p1(), 5,5)

        painter.setPen(Qt.PenStyle.NoPen)
        painter.drawConvexPolygon(self.ok_polygon)
        # painter.drawPolygon(self.ok_polygon)
        # painter.drawText(20,20,str(self._line.angle()))

        # drawlines, polygon, polyline

        # path = QPainterPath()
        # path.moveTo(20, 80)
        # path.lineTo(20, 30)
        # path.cubicTo(80, 0, 50, 50, 80, 80)
        #
        # painter.drawPath(path)
        # painter.drawPath(self.shape())
        # painter.drawRect(self.boundingRect())

        if self._text:
            # painter.setWorldMatrixEnabled(False)
            painter.save()
            painter.setFont(self._font)
            # painter.setPen(self.textPen)
            painter.setPen(self.yaziRengi)
            painter.translate(self._line.center())
            # aci = math.atan2(-self._line.dy(), self._line.dx())
            # aci = aci * 180 / math.pi
            # painter.rotate(- aci)
            if  90 < self._line.angle() < 270:
                painter.rotate(- self._line.angle() - 180)
            else:
                painter.rotate(- self._line.angle())
            painter.translate(-self._line.center())
            # painter.drawText(self.boundingRect(), Qt.AlignCenter | Qt.AlignVCenter, self._text)
            # painter.drawText(self.boundingRect(), self._text, self.painterTextOption)
            painter.drawText(self.painterTextRect, self._text, self.painterTextOption)
            painter.restore()
            # painter.setWorldMatrixEnabled(True)

        if option.state & QStyle.StateFlag.State_Selected or self.cosmeticSelect:

            if self.isActiveItem:
                selectionPenBottom = self.selectionPenBottomIfAlsoActiveItem
            else:
                selectionPenBottom = self.selectionPenBottom

            painter.setBrush(Qt.BrushStyle.NoBrush)

            painter.setPen(selectionPenBottom)
            painter.drawLine(self._line)

            ########################################################################
            # !!! simdilik iptal, gorsel fazlalik olusturmakta !!!
            ########################################################################
            if not self.isPinned and self.isActiveItem:
                # painter.setPen(self.handlePen)
                painter.drawEllipse(self.p1Handle)
                painter.drawEllipse(self.p2Handle)
            ########################################################################

        if option.state & QStyle.StateFlag.State_MouseOver:
            painter.setBrush(Qt.BrushStyle.NoBrush)
            painter.setPen(self.selectionPenBottom)
            painter.drawLine(self._line)

            # painter.setPen(self.selectionPenTop)
            # painter.drawLine(self._line)

            # if self.editMode:
            #     # for a future release, draw points
            #     path = self.path()
            #     painter.setPen(QPen(self.cizgiRengi, 10))
            #     for i in range(path.elementCount()):
            #         painter.drawPoint(QPointF(path.elementAt(i).x, path.elementAt(i).y))

        # # # # # # # debug start - pos() # # # # #
        # painter.setBrush(Qt.NoBrush)
        # p = self.pos()
        # s = self.scenePos()
        # painter.drawText(self.boundingRect(), "{},  {}\n{},  {}".format(p.x(),p.y(),s.x(), s.y()))
        # painter.setPen(QPen(Qt.green,12))
        # painter.drawPoint(self.mapFromScene(self.sceneBoundingRect().center()))
        # # # # painter.drawRect(self.boundingRect())
        # painter.setPen(Qt.green)
        # painter.drawRect(self.boundingRect())
        # painter.drawRect(self.painterTextRect)
        # painter.setPen(Qt.yellow)
        # # painter.drawPolygon(self.rpoly)
        # painter.drawRect(p.x(),p.y(),100,100)
        # painter.setPen(Qt.red)
        # painter.drawRect(s.x(),s.y(),100,100)
        # # painter.drawRect(0,0,15,15)
        # # painter.drawRect(self.path().controlPointRect())
        # painter.setPen(Qt.blue)
        # painter.drawRect(self.boundingRect())
        # # # # # # # debug end - pos() # # # # #

    # ---------------------------------------------------------------------
    def html_dive_cevir(self, html_klasor_kayit_adres, dosya_kopyalaniyor_mu):

        rect = self.sceneBoundingRect()
        xr = rect.left()
        yr = rect.top()
        xs = self.scene().sceneRect().x()
        ys = self.scene().sceneRect().y()
        x = xr - xs
        y = yr - ys

        w = rect.width()
        h = rect.height()

        buffer = QBuffer()
        buffer.open(QIODevice.OpenModeFlag.WriteOnly)

        generator = QSvgGenerator()
        # generator.setFileName("dosya.svg")
        generator.setOutputDevice(buffer)
        # generator.setResolution(72)

        # generator.setSize(QSize(w, h))  # kaymalara sebep oluyor
        generator.setViewBox(QRectF(-w / 2, -h / 2, w, h))
        generator.setTitle(self._kim)
        generator.setDescription("")
        # painter = QPainter(generator)
        painter = QPainter()
        painter.begin(generator)
        painter.save()

        diff = self.scenePos() - rect.center()
        cizilecekLine = QLineF(self._line)
        cizilecekTextRect = QRectF(self.painterTextRect)
        cizilecekOkPolygon = QPolygonF(self.ok_polygon)

        cizilecekLine.translate(diff)
        cizilecekOkPolygon.translate(diff)
        painter.setPen(self._pen)

        painter.translate(diff)
        painter.rotate(self.rotation())
        painter.translate(-diff)
        painter.drawLine(cizilecekLine)
        painter.setBrush(self._pen.color())
        painter.setPen(Qt.PenStyle.NoPen)
        painter.drawConvexPolygon(cizilecekOkPolygon)
        # painter.drawRect(cizilecekTextRect)
        # painter.drawText(cizilecekTextRect, self._text, self.painterTextOption)
        painter.restore()
        if self._text:
            painter.save()
            painter.setFont(self._font)
            # painter.setPen(self.textPen)
            painter.setPen(self.yaziRengi)
            diff = (-cizilecekLine.p1())

            cizilecekTextRect.moveBottomLeft(cizilecekLine.p1())

            painter.translate(-diff)
            painter.rotate((self.rotation() - self._line.angle()) % 360)
            painter.translate(diff)
            # painter.drawText(cizilecekLine.p1(), self._text)
            painter.drawText(cizilecekTextRect, self._text, self.painterTextOption)

            painter.restore()
        painter.end()

        svg_string = buffer.data().data().decode("utf-8")

        # background: rgba{self.arkaPlanRengi.toTuple()};\n
        div_str = f"""
                    <div style="
                     position:absolute;
                     z-index:{int(self.zValue() * 10) if self.zValue() else 0};
                     width:{w}px;
                     height:{h}px;
                     top:{y}px;
                     left:{x}px;" id="{self._kim}">{svg_string}</div>\n
            """
        # return svg_string
        return div_str



# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__date__ = '2/7/21'
__author__ = 'Erdinç Yılmaz'

from PySide6.QtGui import QBrush, QPainter
from PySide6.QtWidgets import QPushButton
from PySide6.QtCore import Qt, Signal
from .shared import kutulu_arkaplan_olustur


########################################################################
class PushButton(QPushButton):

    # ---------------------------------------------------------------------
    def __init__(self, yazi, genislik=None, yukseklik=None, renkArkaplan=None, renkYazi=None, parent=None):
        super(PushButton, self).__init__(yazi, parent)

        self.setFlat(True)

        if genislik:
            self.setFixedWidth(genislik)
        if yukseklik:
            self.setFixedHeight(yukseklik)

        if renkArkaplan or renkYazi:
            self.setAutoFillBackground(True)

            p = self.palette()
            if renkYazi:
                p.setColor(self.foregroundRole(), renkYazi)
            if renkArkaplan:
                p.setColor(self.backgroundRole(), renkArkaplan)
            self.setPalette(p)

        else:
            stil = """
            QPushButton{
                background-color: #eee;
                border: 0;
                padding: 3px;
                border-radius: 3px;
            }
            QPushButton:hover{
                background-color: #def;
            }
            QPushButton:pressed{
                background-color: #cde;
            }
            QPushButton:checked{
                background-color: #ced5e1;
            }
                """
            self.setStyleSheet(stil)


#######################################################################

class PushButtonRenk(QPushButton):
    sagTiklandi = Signal()

    # ---------------------------------------------------------------------
    def __init__(self, yazi, genislik, yukseklik, renk=None, parent=None):
        super(PushButtonRenk, self).__init__(yazi, parent)

        if renk:
            self.renk = renk
        else:
            self.renk = None  # renk yoksa dugmeye ilk tiklandiginda renk secme araci aciliyor
        self.renkGuncelle(renk)
        self.setFixedWidth(genislik)
        self.setFixedHeight(yukseklik)
        kutulu_arkaplan_olustur(self, 3)

    # ---------------------------------------------------------------------
    def renkGuncelle(self, renk):
        self.renk = renk
        self.update()

    # ---------------------------------------------------------------------
    def paintEvent(self, event):
        if self.renk:
            p = QPainter(self)
            # p.begin()
            if self.renk:
                p.fillRect(self.rect(), QBrush(self.renk))
            else:
                p.fillRect(self.rect(), QBrush(Qt.GlobalColor.lightGray))
            # p.setBrush(self.renk)
            # p.drawRect(self.rect())
            p.end()

        # super(PushButtonRenk, self).paintEvent(event)

    # ---------------------------------------------------------------------
    def mousePressEvent(self, event):
        if event.button() == Qt.MouseButton.RightButton:
            self.sagTiklandi.emit()
            event.accept()

        super(PushButtonRenk, self).mousePressEvent(event)



# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__date__ = '2/14/21'
__author__ = 'Erdinç Yılmaz'

# from PySide6.QtGui import
from PySide6.QtWidgets import QWidget, QHBoxLayout, QVBoxLayout
from PySide6.QtCore import Qt, Signal

from .spinBoxlar import DoubleSpinBox
from .slider import Slider


#######################################################################
class SliderDoubleWithDoubleSpinBox(QWidget):
    degerDegisti = Signal(float)

    # ---------------------------------------------------------------------
    def __init__(self, yerlesim=Qt.Orientation.Horizontal, parent=None):
        super(SliderDoubleWithDoubleSpinBox, self).__init__(parent)

        if yerlesim == Qt.Orientation.Horizontal:
            anaLay = QHBoxLayout()
        else:
            anaLay = QVBoxLayout()

        self.setLayout(anaLay)
        anaLay.setContentsMargins(0, 0, 0, 0)

        self.slider = Slider(self)
        self.slider.setOrientation(yerlesim)
        self.dSpinBox = DoubleSpinBox(self)

        anaLay.addWidget(self.slider)
        anaLay.addWidget(self.dSpinBox)
        # anaLay.addStretch()

        self.slider.valueChangedFromSliderGuiNotBySetValue.connect(lambda x: self.degerDegisti.emit(x / 10))
        self.dSpinBox.valueChangedFromDoubleSpinBoxGuiNotBySetValue.connect(lambda x: self.degerDegisti.emit(x))

        self.slider.valueChanged.connect(lambda x: self.dSpinBox.setValue(x / 10))
        self.dSpinBox.valueChanged.connect(lambda x: self.slider.setValue(x * 10))

        """
        self.cizgiKalinligiDSBox.setMinimum(0)
        self.cizgiKalinligiDSBox.setMaximum(100)
        self.cizgiKalinligiDSBox.setSingleStep(0.1)"""

    # # ---------------------------------------------------------------------
    # def act_deger_degisti(self, deger):
    #     self.degerDegisti.emit(deger)

    # ---------------------------------------------------------------------
    def setValue(self, deger):
        self.slider.setValue(deger * 10)
        self.dSpinBox.setValue(deger / 10)

    # ---------------------------------------------------------------------
    def setMinimum(self, deger):
        self.slider.setMinimum(deger)
        self.dSpinBox.setMinimum(deger)

    # ---------------------------------------------------------------------
    def setMaximum(self, deger):
        self.slider.setMaximum(deger * 10)
        self.dSpinBox.setMaximum(deger)

    # ---------------------------------------------------------------------
    def setRange(self, minimum, maximum):
        self.setMinimum(minimum)
        self.setMaximum(maximum)

    # ---------------------------------------------------------------------
    def setSingleStep(self, deger):
        self.dSpinBox.setSingleStep(deger)
        # self.slider.setSingleStep(deger)



# -*- coding: utf-8 -*-
# .

__project_name__ = 'defter'
__date__ = "10/5/23"
__author__ = "E. Y."

from PySide6.QtCore import Qt
from PySide6.QtGui import QColor
from PySide6.QtWidgets import QFrame


########################################################################
class KenarBoyutSol(QFrame):
    # ---------------------------------------------------------------------
    def __init__(self, splitterW, ust_pencere_boyut, renk=QColor(200, 210, 220), parent=None):
        super(KenarBoyutSol, self).__init__(parent)

        self.setAcceptDrops(True)

        # renkBordo = QColor(200, 150, 160)

        # self.renkYazi = QColor(255, 255, 255)
        self.setAutoFillBackground(True)
        self.renkArkaplan = renk
        self.renk_degistir()

        self.setContentsMargins(0, 0, 0, 0)

        # self.setSizePolicy(QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Minimum)

        self.splitterW = splitterW
        self.ust_pencere_boyut = ust_pencere_boyut
        self.min_height = 150
        self.min_width = 30
        self.mouse_start = None
        # self.height_start = self.splitterW.height()
        self.height_start = self.height()
        self.resizing = False
        self.setMouseTracking(True)

        # self.setCursor(Qt.CursorShape.SizeAllCursor)
        self.setCursor(Qt.CursorShape.SizeHorCursor)

        self.show()

    # ---------------------------------------------------------------------
    def renk_degistir(self, renkArkaplan=None):

        p = self.palette()
        # p.setColor(self.foregroundRole(), self.renkYazi)
        # ozellikle boyle yapildi, anlik widgeti baska renge boyayabiliyoruz
        # renk yoksa yine eski rengine don anlamında
        if renkArkaplan:
            p.setColor(self.backgroundRole(), renkArkaplan)
        else:
            p.setColor(self.backgroundRole(), self.renkArkaplan)
        self.setPalette(p)

    # ---------------------------------------------------------------------
    def mousePressEvent(self, event):
        super(KenarBoyutSol, self).mousePressEvent(event)
        self.resizing = True
        self.width_start = self.splitterW.width()
        self.mouse_start = event.globalPos()

    # ---------------------------------------------------------------------
    def mouseMoveEvent(self, event):
        super(KenarBoyutSol, self).mouseMoveEvent(event)
        max_width = self.ust_pencere_boyut.width() // 2
        if self.resizing:
            delta = event.globalPos() - self.mouse_start
            width = self.width_start + delta.x()

            width = max(width, self.min_width)
            width = min(width, max_width)
            # self.splitterW.setFixedWidth(width)

            for i in range(self.splitterW.count()):
                self.splitterW.widget(i).setMinimumWidthVeEskiMinimumSize(width)

    # # ---------------------------------------------------------------------
    # def genislikleri_esitle(self):
    #     if self.splitterW.count() > 1:
    #         for i in range(self.splitterW.count()):
    #             self.splitterW.widget(i).setMinimumWidth(self.splitterW.width())

    # ---------------------------------------------------------------------
    def mouseReleaseEvent(self, event):
        super(KenarBoyutSol, self).mouseReleaseEvent(event)
        self.resizing = False

    # ---------------------------------------------------------------------
    def mouseDoubleClickEvent(self, event):
        super(KenarBoyutSol, self).mouseDoubleClickEvent(event)
        buyult = True
        for i in range(self.splitterW.count()):
            if not self.splitterW.widget(i).kucuk_mu:
                self.splitterW.widget(i).kucult()
                buyult = False
        if buyult:
            for i in range(self.splitterW.count()):
                if self.splitterW.widget(i).kucuk_mu:
                    self.splitterW.widget(i).buyult()

            # self.splitterW.widget(i).setVisible(not self.splitterW.widget(i).isVisible())

    # # ---------------------------------------------------------------------
    # def dragEnterEvent(self, e):
    #     e.accept()
    #
    # # ---------------------------------------------------------------------
    # def dropEvent(self, e):
    #     pos = e.pos()
    #     widget = e.source()
    #
    #     dugmeAdet = self.parent().dugmeLay.count()
    #     if not dugmeAdet:
    #         self.parent().dugmeLay.addWidget(widget)
    #         self.parent().icerikSplitter.addWidget(widget.yw)
    #
    #     else:
    #         birak = False
    #         for n in range(dugmeAdet):
    #             w = self.parent().dugmeLay.itemAt(n).widget()
    #             birak = pos.y() < w.y() + w.size().height() // 2
    #
    #             if birak:
    #                 self.parent().dugmeLay.insertWidget(n-1, widget)
    #                 self.parent().icerikSplitter.insertWidget(n-1, widget.yw)
    #                 # self.dugmeTasindi.emit(widget)
    #                 break
    #
    #     e.accept()


########################################################################
class KenarBoyutSag(KenarBoyutSol):
    # ---------------------------------------------------------------------
    def __init__(self, splitterW, ust_pencere_boyut, renk=QColor(200, 210, 220), parent=None):
        super(KenarBoyutSag, self).__init__(splitterW, ust_pencere_boyut, renk, parent)

    # ---------------------------------------------------------------------
    def mouseMoveEvent(self, event):
        super(KenarBoyutSag, self).mouseMoveEvent(event)
        max_width = self.ust_pencere_boyut.width() // 2
        if self.resizing:
            delta = event.globalPos() - self.mouse_start
            width = self.width_start - delta.x()

            width = max(width, self.min_width)
            width = min(width, max_width)
            # self.splitterW.setFixedWidth(width)

            for i in range(self.splitterW.count()):
                self.splitterW.widget(i).setMinimumWidthVeEskiMinimumSize(width)
